@@ -2,7 +2,7 @@
                           fitdlg.cpp  -  description
                              -------------------
     begin                : Tue May 25 1999
-    copyright            : (C) 2001 by Werner Stille
+    copyright            : (C) 2002 by Werner Stille
     email                : stille@uni-freiburg.de
  ***************************************************************************/
 
@@ -28,6 +28,7 @@
 #include <qdir.h>
 #include <qlayout.h>
 #include <qtable.h>
+#include <qtimer.h>
 #include <kapp.h>
 #include <klocale.h>
 #include <kurl.h>
@@ -36,22 +37,23 @@
 #include <kconfig.h>
 #include <sourcewidget.h>
 #include <kmessagebox.h>
-#include <kio/job.h>
-#include <kio/netaccess.h>
 #include <kglobal.h>
 #include "fitdlg.h"
 #include "lmfit.h"
 #include "kpldoc.h"
+#include "frameitem.h"
 #include "arrayitem.h"
 #include "funitem.h"
 #include "utils.h"
 #include "kpldoubleedit.h"
 #include "kplspinbox.h"
+#include "residualsdlg.h"
+#include "kplgraph.h"
 
-CorrDlg::CorrDlg(QWidget* parent, int np, const int* ip, const double* corr) :
-  QDialog(parent, 0, true)
+CorrDlg::CorrDlg(QWidget* _parent, int np, const int* ip, const double* corr) :
+  KDialog(_parent, 0, true)
 {
-  setCaption(i18n("Parameter correlation coefficients"));
+  setPlainCaption(i18n("Parameter correlation coefficients"));
   QTable* table = new QTable(np, np, this);
   table->setMinimumSize(100, 40);
   QString s;
@@ -77,47 +79,41 @@ CorrDlg::~CorrDlg()
 {
 }
 
-ErrModDlg::ErrModDlg(QWidget* parent, KplDoc* model,
-                     Kpl::DataErrorStruct* err0) : QDialog(parent, 0, true),
- m(model), err(err0)
+ErrModDlg::ErrModDlg(QWidget* _parent, KplDoc* model,
+                     Kpl::DataErrorStruct* err0) :
+ KDialogBase(Plain, i18n("Error model function"), Help | Ok | Cancel, Ok,
+            _parent, 0, true, true), m(model), err(err0)
 {
   errt = new Kpl::DataErrorStruct(*err);
-  setCaption(i18n("Error model Function"));
-  QVBoxLayout* vbox = new QVBoxLayout(this, 11, 6);
-  QGridLayout* grid = new QGridLayout(vbox, 3, 2);
-  grid->addWidget(new QLabel(i18n("Library"), this), 0, 0);
+  QFrame* frame = plainPage();
+  QGridLayout* grid = new QGridLayout(frame, 3, 2, 0, spacingHint());
+  grid->addWidget(new QLabel(i18n("Library"), frame), 0, 0);
   QHBoxLayout* hbox = new QHBoxLayout();
   grid->addLayout(hbox, 0, 1);
-  hbox->addWidget(fileName = new QLineEdit(err->errModPath, this));
-  QPushButton* b = new QPushButton("...", this);
+  hbox->addWidget(fileName = new QLineEdit(err->errModPath, frame));
+  QPushButton* b = new QPushButton("...", frame);
   b->setFixedWidth(28);
   hbox->addWidget(b);
   connect(b, SIGNAL(clicked()), SLOT(slotFile()));
-  grid->addWidget(new QLabel(i18n("Function"), this), 1, 0);
+  grid->addWidget(new QLabel(i18n("Function"), frame), 1, 0);
   grid->addLayout(hbox = new QHBoxLayout(), 1, 1);
-  hbox->addWidget(func = new QLineEdit(err->errModName, this));
-  hbox->addWidget(b = new QPushButton("...", this));
+  hbox->addWidget(func = new QLineEdit(err->errModName, frame));
+  hbox->addWidget(b = new QPushButton("...", frame));
   b->setFixedWidth(28);
+#if KDE_VERSION == 303
+  connect(this, SIGNAL(helpClicked()), SLOT(slotHelp2()));
+#else
+  setHelp("ERRORMODEL");
+#endif
   connect(b, SIGNAL(clicked()), SLOT(slotFunc()));
-  grid->addWidget(new QLabel(i18n("Parameter"), this), 2, 0);
+  grid->addWidget(new QLabel(i18n("Parameter"), frame), 2, 0);
   grid->addLayout(hbox = new QHBoxLayout(), 2, 1);
-  hbox->addWidget(b = new QPushButton(i18n("Edit"), this));
+  hbox->addWidget(b = new QPushButton(i18n("Edit"), frame));
   connect(b, SIGNAL(clicked()), SLOT(slotEditParameter()));
-  hbox->addWidget(b = new QPushButton(i18n("Load"), this));
+  hbox->addWidget(b = new QPushButton(i18n("Load"), frame));
   connect(b, SIGNAL(clicked()), SLOT(slotGetParameter()));
-  hbox->addWidget(b = new QPushButton(i18n("Save"), this));
+  hbox->addWidget(b = new QPushButton(i18n("Save"), frame));
   connect(b, SIGNAL(clicked()), SLOT(slotSaveParameter()));
-  KSeparator* hl = new KSeparator(KSeparator::HLine, this);
-  vbox->addWidget(hl);
-  hbox = new QHBoxLayout(vbox);
-  hbox->addWidget(b = new QPushButton(i18n("Help"), this));
-  connect(b, SIGNAL(clicked()), SLOT(slotHelp()));
-  hbox->addItem(new QSpacerItem(20, 10));
-  hbox->addWidget(b = new QPushButton(i18n("OK"), this));
-  b->setDefault(true);
-  connect(b, SIGNAL(clicked()), SLOT(slotOK()));
-  hbox->addWidget(b = new QPushButton(i18n("Cancel"), this));
-  connect(b, SIGNAL(clicked()), SLOT(reject()));
 }
 
 ErrModDlg::~ErrModDlg()
@@ -159,73 +155,81 @@ void ErrModDlg::slotSaveParameter()
   Utils::saveFunPar(this, errt->pErrMod, m);
 }
 
-void ErrModDlg::slotOK()
+void ErrModDlg::slotOk()
 {
   errt->errModPath = fileName->text();
   errt->errModName = func->text();
   void *hErrMod = 0;
   double (*fkt)(double, const double *);
-  if (Utils::getFuncAddr(errt->errModPath, errt->errModName, &hErrMod, &fkt))
+  if (FunItem::getFuncAddr(errt->errModPath, errt->errModName, &hErrMod, &fkt))
     return;
   dlclose(hErrMod);
   *err = *errt;
   accept();
 }
 
-void ErrModDlg::slotHelp()
+void ErrModDlg::slotHelp2()
 {
-  kapp->invokeHelp("ERRORMODEL");
+#if KDE_VERSION == 303
+  kapp->startServiceByDesktopName("konqueror",
+                                  QString("help:/kpl/fit.html#errormodel"),
+                                  0, 0, 0, "", true);
+#endif
 }
 
-FitDlg::FitDlg(QWidget* parent, KplDoc* model, QList<ArrayItem>* ad0,
-               QList<FunItem>* fd0, int mode) : QDialog(parent, 0, true),
- dlgMode(mode), m(model), ad(ad0), fd(fd0), lm(0), sw(0)
+FitDlg::FitDlg(QWidget* _parent, KplDoc* model, QList<ArrayItem>* ad0,
+               QList<FunItem>* fd0, int mode) :
+ KDialogBase(Plain, i18n("Parameter fit"),
+             Help | Ok | Apply | Cancel | User1 | User2,
+             Ok, _parent, 0, true, true, i18n("&Start"), i18n("&Residuals")),
+ running(false), dlgMode(mode), m(model), ad(ad0), fd(fd0), sw(0)
 {
-  aut = new Kpl::AutoStruct(m->aut);
+  aut = new Kpl::AutoStruct(*m->options());
   config = KGlobal::config();
   err = new Kpl::DataErrorStruct[ad->count()];
   int i;
   QString s;
   ArrayItem* a;
   for (i = 0; i < (int) ad->count(); i++) {
-    err[i] = m->aut.err;
+    err[i] = aut->err;
     s.sprintf("DataErrors%i", i);
-    config->setGroup(s);
-    err[i].errModPath = config->readEntry("ErrorModelPath",
-                                          m->aut.err.errModPath);
-    err[i].errModName = config->readEntry("ErrorModelName",
-                                          m->aut.err.errModName);
-    QStringList list = config->readListEntry("ErrorModelParam", ' ');
-    int cnt = list.count();
-    for (int j = 0; j < KPL_NPMAX; j++)
-      err[i].pErrMod[j] = (j < cnt) ? list[j].toDouble() : 0.0;
-    a = ad->at(i);
-    if (a->ie < a->ncols)
-      err[i].fitErrCol = config->readBoolEntry("FitErrorColumn",
-                                               m->aut.err.fitErrCol);
-    else
-      err[i].fitErrCol = false;
+    if (config->hasGroup(s)) {
+      config->setGroup(s);
+      err[i].errModPath = config->readEntry("ErrorModelPath",
+                                            aut->err.errModPath);
+      err[i].errModName = config->readEntry("ErrorModelName",
+                                            aut->err.errModName);
+      QStringList list = config->readListEntry("ErrorModelParam", ' ');
+      int cnt = list.count();
+      for (int j = 0; j < KPL_NPMAX; j++)
+        err[i].pErrMod[j] = (j < cnt) ? list[j].toDouble() : 0.0;
+      a = ad->at(i);
+      if (a->ie < a->ncols)
+        err[i].fitErrCol = config->readBoolEntry("FitErrorColumn",
+                                                 aut->err.fitErrCol);
+      else
+        err[i].fitErrCol = false;
+    }
   }
   FunItem* ff = fd->first();
   for (i = 0; i < KPL_NPMAX; i++) {
     m->pLast[i] = ff->py[i];
     m->pErr[i] = 0.0;
   }
+  lm = new LMFit(this);
   if (dlgMode & ShowDlg) {
-    config->setGroup("FitDialog");
-    resize(config->readNumEntry("Width", 530),
-           config->readNumEntry("Height", 560));
-    setCaption(i18n("Parameter fit"));
+    Utils::setSize(this, "FitDialog");
     QHBoxLayout* hbox = 0;
     QVBoxLayout* vbox;
-    if (m->aut.showSource) {
-      hbox = new QHBoxLayout(this, 11, 6);
-      vbox = new QVBoxLayout(hbox, 6);
+    QFrame* frame = plainPage();
+    if (m->options()->showSource) {
+      hbox = new QHBoxLayout(frame, 11, spacingHint());
+      vbox = new QVBoxLayout(hbox);
     } else
-      vbox = new QVBoxLayout(this, 11, 6);
-    QGroupBox* g = new QGroupBox(0, Qt::Vertical, i18n("Parameter"), this);
+      vbox = new QVBoxLayout(frame, 11, spacingHint());
+    QGroupBox* g = new QGroupBox(0, Qt::Vertical, i18n("Parameter"), frame);
     vbox->addWidget(g);
-    QVBoxLayout* vbox2 = new QVBoxLayout(g->layout(), 6);
+    QVBoxLayout* vbox2 = new QVBoxLayout(g->layout(), spacingHint());
     int nGr = 1 + (KPL_NPMAX - 1) / 10;
     QGridLayout* grid = new QGridLayout(vbox2, QMIN(KPL_NPMAX, 10),
                                         5 * nGr - 1);
@@ -237,8 +241,8 @@ FitDlg::FitDlg(QWidget* parent, KplDoc* model, QList<ArrayItem>* ad0,
       grid->addWidget(enFit[i] = new QCheckBox(s.sprintf("p%i", i), g, "f"),
                       iy, ix);
       enFit[i]->setChecked(aut->fitPar[i]);
-      grid->addWidget(par[i] = new QLineEdit(QString::number(m->pLast[i],
-        m->aut.format, m->aut.prec), g), iy, ix + 1);
+      grid->addWidget(par[i] = new QLineEdit(m->number(m->pLast[i]), g),
+                      iy, ix + 1);
       par[i]->setMaximumWidth(80);
       grid->addWidget(new QLabel("\261", g), iy, ix + 2);
       grid->addWidget(epar[i] = new QLabel("0", g), iy, ix + 3);
@@ -247,7 +251,7 @@ FitDlg::FitDlg(QWidget* parent, KplDoc* model, QList<ArrayItem>* ad0,
       epar[i]->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding,
                                          QSizePolicy::Fixed));
     }
-    QHBoxLayout* hbox2 = new QHBoxLayout(vbox2, 6);
+    QHBoxLayout* hbox2 = new QHBoxLayout(vbox2, spacingHint());
     hbox2->addWidget(nonLin = new QCheckBox(i18n("Nonlinear fit"), g));
     nonLin->setChecked(aut->fitNonLin);
     widgetList.append(nonLin);
@@ -261,69 +265,63 @@ FitDlg::FitDlg(QWidget* parent, KplDoc* model, QList<ArrayItem>* ad0,
     hbox2->addWidget(showCorr = new QPushButton(i18n("Correlations"), g));
     showCorr->setEnabled(false);
     connect(showCorr, SIGNAL(clicked()), SLOT(slotCorr()));
-    g = new QGroupBox(0, Qt::Horizontal, i18n("Data errors"), this);
-    hbox2 = new QHBoxLayout(g->layout(), 6);
+    g = new QGroupBox(0, Qt::Horizontal, i18n("Data errors"), frame);
+    hbox2 = new QHBoxLayout(g->layout(), spacingHint());
     hbox2->addWidget(new QLabel(i18n("Array"), g));
     a = ad->at(0);
-    QHBoxLayout* hbox3 = new QHBoxLayout(hbox2, 6);
-    hbox3->addWidget(sArr = new KplSpinBox(0, ad->count() - 1, 1, g));
-    connect(sArr, SIGNAL(valueChanged(int)), SLOT(updArray(int)));
+    hbox2->addWidget(sArr = new KplSpinBox(0, ad->count() - 1, 1, g));
     sArr->setValue(0);
-    hbox3->addWidget(lArr = new QLabel(g));
+    hbox2->addWidget(lArr = new QLabel(g));
     lArr->setFrameStyle(QFrame::Panel | QFrame::Sunken);
     hbox2->addWidget(errCol = new QCheckBox(i18n("Error column"), g));
-    connect(errCol, SIGNAL(toggled(bool)), SLOT(errColToggled(bool)));
     hbox2->addItem(new QSpacerItem(10, 10, QSizePolicy::Expanding));
     hbox2->addWidget(errMod = new QPushButton(i18n("Model"), g));
     connect(errMod, SIGNAL(clicked()), SLOT(slotErrMod()));
     vbox->addWidget(g);
-    hbox2 = new QHBoxLayout(vbox, 6);
-    hbox2->addWidget(new QLabel(i18n("Maximum iterations"), this));
-    hbox2->addWidget(sItmax = new KplSpinBox(1, 200, 1, this));
+    hbox2 = new QHBoxLayout(vbox, spacingHint());
+    hbox2->addWidget(new QLabel(i18n("Maximum iterations"), frame));
+    hbox2->addWidget(sItmax = new KplSpinBox(1, 200, 1, frame));
     sItmax->setValue(aut->fitMaxIt);
     widgetList.append(sItmax);
     hbox2->addItem(new QSpacerItem(10, 10, QSizePolicy::Expanding));
-    hbox2->addWidget(new QLabel(i18n("Tolerance"), this));
-    hbox2->addWidget(eTol = new KplDoubleEdit(aut->fitTol, 0.0, 1.0, this,
+    hbox2->addWidget(new QLabel(i18n("Tolerance"), frame));
+    hbox2->addWidget(eTol = new KplDoubleEdit(aut->fitTol, 0.0, 1.0, frame,
                                               "%.6lg"));
-    connect(nonLin, SIGNAL(toggled(bool)), SLOT(enableNonLin(bool)));
-    enableNonLin(aut->fitNonLin);
     widgetList.append(eTol);
-    vbox->addWidget(results = new QListBox(this));
+    vbox->addWidget(results = new QListBox(frame));
     results->setMinimumHeight(40);
-    hbox2 = new QHBoxLayout(vbox, 6);
-    hbox2->addWidget(b = new QPushButton(i18n("Help"), this));
-    connect(b, SIGNAL(clicked()), SLOT(slotHelp()));
-    hbox2->addWidget(b = new QPushButton(i18n("Start"), this));
-    connect(b, SIGNAL(clicked()), SLOT(slotFit()));
-    hbox2->addWidget(b = new QPushButton(i18n("OK"), this));
-    b->setDefault(true);
-    connect(b, SIGNAL(clicked()), SLOT(slotOK()));
-    widgetList.append(b);
-    hbox2->addWidget(b = new QPushButton(i18n("Apply"), this));
-    connect(b, SIGNAL(clicked()), SLOT(slotApply()));
-    widgetList.append(b);
-    hbox2->addWidget(b = new QPushButton(i18n("Cancel"), this));
-    connect(b, SIGNAL(clicked()), SLOT(slotCancel()));
-    if (m->aut.showSource) {
-      hbox->addWidget(sw = new SourceWidget(this, 0), 1);
+    widgetList.append(actionButton(Ok));
+    widgetList.append(actionButton(Apply));
+    widgetList.append(actionButton(User2));
+    enableButton(User2, false);
+    if (m->options()->showSource) {
+      hbox->addWidget(sw = new SourceWidget(frame, 0), 1);
       sw->setMinimumWidth(200);
     }
+#if KDE_VERSION == 303
+    connect(this, SIGNAL(helpClicked()), SLOT(slotHelp2()));
+#else
+    setHelp("SEC-FIT");
+#endif
+    connect(nonLin, SIGNAL(toggled(bool)), SLOT(enableNonLin(bool)));
+    connect(errCol, SIGNAL(toggled(bool)), SLOT(errColToggled(bool)));
+    connect(sArr, SIGNAL(valueChanged(int)), SLOT(updArray(int)));
+    connect(lm, SIGNAL(updateMessage(const QString &)),
+            SLOT(slotMessage(const QString &)));
+    enableNonLin(aut->fitNonLin);
     updArray(0);
-    if (m->aut.showSource)
+    if (m->options()->showSource)
       QTimer::singleShot(0, sw, SLOT(slotHighlight()));
   }
 }
 
 FitDlg::~FitDlg()
 {
-  if (dlgMode & ShowDlg) {
-    config->setGroup("FitDialog");
-    config->writeEntry("Width", width());
-    config->writeEntry("Height", height());
-  }
+  if (dlgMode & ShowDlg)
+    Utils::saveSize(this, "FitDialog");
   delete [] err;
   delete aut;
+  delete lm;
 }
 
 void FitDlg::getValues(bool ok)
@@ -337,7 +335,7 @@ void FitDlg::getValues(bool ok)
       aut->fitMaxIt = sItmax->interpretedValue();
       aut->fitTol = eTol->value();
     }
-    m->aut = *aut;
+    m->setOptions(aut);
     QString s;
     for (j = 0; j < (int) fd->count(); j++) {
       s.sprintf("DataErrors%i", j);
@@ -347,8 +345,7 @@ void FitDlg::getValues(bool ok)
       config->writeEntry("ErrorModelName", err[j].errModName);
       QStrList s1;
       for (i = 0; i < KPL_NPMAX; i++)
-        s1.insert(i, QString::number(err[j].pErrMod[i], aut->format,
-                                     aut->prec));
+        s1.insert(i, m->number(err[j].pErrMod[i]));
       config->writeEntry("ErrorModelParam", s1, ' ');
     }
   }
@@ -372,7 +369,7 @@ void FitDlg::slotGetParameter()
 {
   Utils::getPar(this, m->pLast, m);
   for (int i = 0; i < KPL_NPMAX; i++) {
-    par[i]->setText(QString::number(m->pLast[i], m->aut.format, m->aut.prec));
+    par[i]->setText(m->number(m->pLast[i]));
     m->pErr[i] = 0.0;
     epar[i]->setText("0");
   }
@@ -404,7 +401,7 @@ void FitDlg::updArray(int i)
                 QString::number(a->iy) + " " + QString::number(a->ie));
   errCol->setChecked(err[i].fitErrCol);
   errCol->setEnabled(a->ie < a->ncols);
-  if (m->aut.showSource) {
+  if (m->options()->showSource) {
     FunItem* ff = fd->at(i);
     QFileInfo fi(ff->pathy.path());
     int pp = ff->namey.find("__");
@@ -422,14 +419,14 @@ void FitDlg::updArray(int i)
   }
 }
 
-void FitDlg::slotOK()
+void FitDlg::slotOk()
 {
   getValues(true);
 }
 
 void FitDlg::slotCancel()
 {
-  if (lm)
+  if (running)
     lm->userBreak = true;
   else
     reject();
@@ -440,14 +437,23 @@ void FitDlg::slotApply()
   getValues(false);
 }
 
-void FitDlg::slotHelp()
+
+void FitDlg::slotHelp2()
 {
-  kapp->invokeHelp("SEC-FIT");
+#if KDE_VERSION == 303
+  kapp->startServiceByDesktopName("konqueror", QString("help:/kpl/fit.html"),
+                                  0, 0, 0, "", true);
+#endif
 }
 
-void FitDlg::slotFit()
+void FitDlg::slotUser1()
 {
-  for (FunItem *f = fd->first(); f; f = fd->next())
+  fit();
+}
+
+void FitDlg::fit()
+{
+  for (FunItem* f = fd->first(); f; f = fd->next())
     if (!f->fkty) {
       KMessageBox::error(this, i18n("no function address!"));
       return;
@@ -487,8 +493,8 @@ void FitDlg::slotFit()
   ArrayItem* a;
   for (a = ad->first(); a; a = ad->next())
     n += a->n;
-  lm = new LMFit(this);
-  lm->sig = new double[n];
+  fvec.resize(n);
+  lm->sig.resize(n);
   int j = 0;
   int k;
   for (k = 0; k < (int) ad->count(); k++) {
@@ -499,13 +505,9 @@ void FitDlg::slotFit()
     } else {
       double (*fktErrMod)(double, const double*);
       void* hErrMod;
-      if (Utils::getFuncAddr(err[k].errModPath, err[k].errModName, &hErrMod,
-                             &fktErrMod)) {
-        delete [] lm->sig;
-        delete lm;
-        lm = 0;
+      if (FunItem::getFuncAddr(err[k].errModPath, err[k].errModName, &hErrMod,
+                               &fktErrMod))
         return;
-      }
       for (i = 0; i < a->n; i++)
         lm->sig[j++] = fktErrMod(a->x[a->iy][a->istart + i], err[k].pErrMod);
       dlclose(hErrMod);
@@ -515,9 +517,8 @@ void FitDlg::slotFit()
   lm->fd = fd;
   lm->bFit = aut->fitPar;
   lm->p = m->pLast;
-  double* wa = (double *) calloc((6 + np + n) * np + 2 * n, sizeof(double));
-  double* fvec = wa + 3 * np + n;
-  double* guess = fvec + n;
+  double* wa = (double*) calloc((6 + np + n) * np + n, sizeof(double));
+  double* guess = wa + 3 * np + n;
   double* diag = guess + np;
   double* fjac = diag + np;
   double* qtf = fjac + n * np;
@@ -533,15 +534,14 @@ void FitDlg::slotFit()
       par[i]->setEnabled(false);
       enFit[i]->setEnabled(false);
     }
-    connect(lm, SIGNAL(updateMessage(const QString &)),
-            SLOT(slotMessage(const QString &)));
   }
   kapp->processEvents();
-  bool aborted = false;
+  bool aborted = lm->userBreak = false;
+  running = true;
   if (nLin) {
     int info = 0;
     int nfev;
-    lm->lmdif(n, np, guess, fvec, aut->fitTol, aut->fitTol, 0.0,
+    lm->lmdif(n, np, guess, fvec.data(), aut->fitTol, aut->fitTol, 0.0,
               (np + 1) * aut->fitMaxIt, 0.0, diag, 1, 100.0, 1, &info, &nfev,
               fjac, n, ip, qtf, wa, wa + np, wa + 2 * np, wa + 3 * np);
     if (info >= 8)
@@ -582,16 +582,17 @@ void FitDlg::slotFit()
       slotMessage(s);
     }
   } else {
-    aborted = lm->linFit(n, np, guess, fvec, fjac, ip, qtf, wa, wa + np,
+    aborted = lm->linFit(n, np, guess, fvec.data(), fjac, ip, qtf, wa, wa + np,
                          wa + 2 * np, wa + 3 * np);
   }
+  running = false;
   if (!aborted) {
     j = 0;
     for (i = 0; i < KPL_NPMAX; i++)
       if (aut->fitPar[i])
         m->pLast[i] = guess[j++];
-    m->chisq = enorm(n, fvec);
-    m->chisq *= m->chisq;
+    double chi = LMFit::enorm(n, fvec);
+    m->chisq = chi * chi;
     for (i = 0; i < np; i++) {
       for (j = 0; j < np; j++) {
         alpha[ip[i] + ip[j] * np] = 0.0;
@@ -600,7 +601,7 @@ void FitDlg::slotFit()
             alpha[ip[i] + ip[j] * np] += fjac[i * n + k] * fjac[j * n + k];
       }
     }
-    minv(alpha, corr, np, wa, ip);
+    LMFit::minv(alpha, corr, np, wa, ip);
     int ij = 0;
     for (i = 0; i < np; i++) {
       diag[i] = sqrt(corr[ij]);
@@ -616,7 +617,7 @@ void FitDlg::slotFit()
     int ny = n - np;
     double q = 0.0;
     if (ny) {
-      q = igamc(0.5 * ny, 0.5 * m->chisq);
+      q = LMFit::igamc(0.5 * ny, 0.5 * m->chisq);
       double temp = sqrt(m->chisq / ny);
       for (i = 0; i < np; i++)
         diag[i] *= temp;
@@ -629,10 +630,8 @@ void FitDlg::slotFit()
       } else
         m->pErr[i] = 0.0;
       if (dlgMode & ShowDlg) {
-        par[i]->setText(QString::number(m->pLast[i], m->aut.format,
-                        m->aut.prec));
-        epar[i]->setText(QString::number(m->pErr[i], m->aut.format,
-                         m->aut.prec));
+        par[i]->setText(m->number(m->pLast[i]));
+        epar[i]->setText(m->number(m->pErr[i]));
       }
       if (dlgMode & Follow)
         aut->pFit[i] = m->pLast[i];
@@ -664,10 +663,7 @@ void FitDlg::slotFit()
       enFit[i]->setEnabled(true);
     }
   }
-  delete [] lm->sig;
   free(wa);
-  delete lm;
-  lm = 0;
 }
 
 void FitDlg::slotErrMod()
@@ -676,6 +672,70 @@ void FitDlg::slotErrMod()
   dlg.exec();
 }
 
+void FitDlg::slotUser2()
+{
+  QList<KplItem> items;
+  items.setAutoDelete(true);
+  FrameItem* fd = new FrameItem();
+  fd->gridmode = KplGraph::GridWithLabels;
+  fd->sx = "x";
+  fd->sy = i18n("Residuals");
+  items.append(fd);
+  int j = 0;
+  for (ArrayItem* a = ad->first(); a; a = ad->next()) {
+    ArrayItem* ar = new ArrayItem(true, a->symb, "0", 1.0, 1.0, 0, 1, 2,
+                                  0, a->n, 1, "", true);
+    ar->color = a->color;
+    for (int i = 0; i < a->n; i++) {
+      ar->x[0][i] = a->x[a->ix][a->istart + i];
+      ar->x[1][i] = fvec[j] * lm->sig[j];
+      ar->x[2][i] = lm->sig[j++];
+    }
+    items.append(ar);
+  }
+  double fxt, fyt;
+  double fx = 1.0;
+  double fy = 1.0;
+  int iext, ieyt;
+  fd->iex = fd->iey = 999;
+  for (KplItem* itm = items.at(1); itm; itm = items.next()) {
+    iext = fd->iex;
+    ieyt = fd->iey;
+    itm->expoItem(&iext, &ieyt, &fxt, &fyt);
+    if (iext < fd->iex) {
+      fd->iex = iext;
+      fx = fxt;
+    }
+    if (ieyt < fd->iey) {
+      fd->iey = ieyt;
+      fy = fyt;
+    }
+  }
+  double xmin = 1.0e300;
+  double ymin = xmin;
+  double xmax = -xmin;
+  double ymax = xmax;
+  for (KplItem *itm = items.at(1); itm; itm = items.next()) {
+    fd->xmi = xmin;
+    fd->ymi = ymin;
+    fd->xma = xmax;
+    fd->yma = ymax;
+    itm->minMax(&fd->xmi, &fd->xma, &fd->ymi, &fd->yma);
+    xmin = QMIN(xmin, fd->xmi);
+    xmax = QMAX(xmax, fd->xma);
+    ymin = QMIN(ymin, fd->ymi);
+    ymax = QMAX(ymax, fd->yma);
+  }
+  Utils::autoSc(&fd->xmi, &fd->xma, &fd->xtic, &fd->mticx, &fd->ndigx,
+                xmin, xmax, fx, 0.2, false);
+  Utils::autoSc(&fd->ymi, &fd->yma, &fd->ytic, &fd->mticy, &fd->ndigy,
+                ymin, ymax, fy, 0.3, false);
+  for (KplItem* itm = items.at(1); itm; itm = items.next())
+    itm->normalize(fx, fy);
+  ResidualsDlg dlg(this, m, &items);
+  dlg.exec();
+}
+
 void FitDlg::slotCorr()
 {
   CorrDlg dlg(this, np, ip, corr);
