@@ -6,7 +6,7 @@
     data sets and functions.
 
     begin                : Sun Apr 25 1999
-    copyright            : (C) 2001 by Werner Stille
+    copyright            : (C) 2002 by Werner Stille
     email                : stille@uni-freiburg.de
  ***************************************************************************/
 
@@ -22,6 +22,7 @@
 #include <math.h>
 #include <qpaintdevicemetrics.h>
 #include <qpixmap.h>
+#include <kapp.h>
 #include "kgraph.h"
 
 //#define NO_CLIPPING
@@ -32,7 +33,10 @@ KGraph::KGraph() : minx(0), maxx(0), miny(0), maxy(0), kxcur(0), kycur(0),
  ndigx(0), ndigy(0), minx0(0), miny0(0), pFactor(0), textdir(0.0),
  scalx(30.0), scaly(-30.0), xmin(0.0), xmax(15.0), ymin(0.0), ymax(10.0),
  xcur(0.0), ycur(0.0), dlen(15.0), dhite(10.0), xscal(30.0), yscal(-30.0),
- relang(0.0), relsin(0.0), relcos(1.0), dcur(0.0), factor(1.0)
+ relang(0.0), relsin(0.0), relcos(1.0), dcur(0.0), factor(1.0),
+ pen1(black, 0, SolidLine, RoundCap, RoundJoin),
+ pen2(black, 0, SolidLine, RoundCap, RoundJoin),
+ pen3(black, 0, SolidLine, RoundCap, RoundJoin)
 {
   memset(dash, 0, sizeof(dash));
 }
@@ -56,13 +60,13 @@ bool KGraph::i2cm(const QPoint& p, double* xcm, double* ycm) const
   return ((p.x() < minx) || (p.x() > maxx) || (p.y() > miny) || (p.y() < maxy));
 }
 
-void KGraph::i2r(const QPoint& p, double fxn, double fyn,
+void KGraph::i2r(const QPoint& p, double _fxn, double _fyn,
                  double* xr, double* yr) const
 {
-  *xr = ((p.x() - minx) / xscal + xmin) / fxn;
+  *xr = ((p.x() - minx) / xscal + xmin) / _fxn;
   if (logx)
     *xr = pow(10, *xr);
-  *yr = ((p.y() - miny) / yscal + ymin) / fyn;
+  *yr = ((p.y() - miny) / yscal + ymin) / _fyn;
   if (logy)
     *yr = pow(10, *yr);
 }
@@ -73,10 +77,10 @@ QPoint KGraph::fromcm(double xcm, double ycm) const
                 qRound(ycm * scaly * factor) + miny0);
 }
 
-void KGraph::cm2r(double xcm, double ycm, double fxn, double fyn,
+void KGraph::cm2r(double xcm, double ycm, double _fxn, double _fyn,
                   double* xr, double* yr) const
 {
-  i2r(fromcm(xcm, ycm), fxn, fyn, xr, yr);
+  i2r(fromcm(xcm, ycm), _fxn, _fyn, xr, yr);
 }
 
 void KGraph::move(double x, double y)
@@ -141,19 +145,22 @@ void KGraph::line(double x1, double y1, double x2, double y2)
 
 void KGraph::plInit(const QPaintDevice* pd, int offLeft, int offTop)
 {
-  pFactor = 1;
   QPaintDeviceMetrics pdm(pd);
   begin(pd);
   eraseRect(window());
   if (pd->devType() == QInternal::Printer) {
     devType = Printer;
-    QRect r = window();
     pFactor = 10;
+    QRect r = window();
     r.setSize(pFactor * r.size());
     setWindow(r);
+    fDpi = 1.0;
+  } else {
+    pFactor = 1;
+    fDpi = 75.0 / pdm.logicalDpiY();
   }
-  scalx = 10.0 * pFactor * pdm.width() / pdm.widthMM();
-  scaly = -10.0 * pFactor * pdm.height() / pdm.heightMM();
+  scalx = pFactor * pdm.logicalDpiX() / 2.54;
+  scaly = -pFactor * pdm.logicalDpiY() / 2.54;
   minx0 = minx = -pFactor * offLeft;
   maxx = pFactor * pdm.width() + minx;
   miny0 = miny = pFactor * (pdm.height() + offTop);
@@ -182,9 +189,13 @@ void KGraph::plInit(const QPaintDevice* pd, int offLeft, int offTop)
   setPen(pen3);
   QFont fnt = font();
   fnt.setFamily(face[0]);
-  fnt.setPointSize(QMAX(qRound(0.0355 * maxx), 1));
+  fnt.setPointSize(QMAX(qRound(0.0355 * fDpi * maxx), 1));
   setFont(fnt);
   move(xcur, ycur);
+#ifndef NO_CLIPPING
+  setClipRect(-offLeft, offTop, pdm.width(), pdm.height());
+  setClipping(false);
+#endif
 }
 
 void KGraph::format(double x, double y)
@@ -201,7 +212,7 @@ void KGraph::format(double x, double y)
   xcur = xmin;
   ycur = ymin;
 #ifndef NO_CLIPPING
-  setClipRect(0, maxy / pFactor, (maxx - minx) / pFactor,
+  setClipRect(minx / pFactor, maxy / pFactor, (maxx - minx) / pFactor,
               (miny - maxy) / pFactor);
   setClipping(false);
 #endif
@@ -239,7 +250,7 @@ double KGraph::setRelSize(double r)
   pen3.setWidth(qRound(0.00295 * relSize * (maxx - minx)));
   setPen(pen3);
   QFont fnt = font();
-  fnt.setPointSize(QMAX(qRound(0.0355 * relSize * (maxx - minx)), 1));
+  fnt.setPointSize(QMAX(qRound(0.0355 * fDpi * relSize * (maxx - minx)), 1));
   setFont(fnt);
   return old;
 }
@@ -258,17 +269,17 @@ void KGraph::Window(double xmi, double w, double ymi, double h)
   ycur = ymin;
   move(xcur, ycur);
   dcur = 0.0;
-  int dx = qRound(0.00416667 * relSize * (maxx - minx));
-  pen1.setWidth(dx);
+  int _dx = qRound(0.00416667 * relSize * (maxx - minx));
+  pen1.setWidth(_dx);
   pen2.setWidth(qRound(0.00208333 * relSize * (maxx - minx)));
   pen3.setWidth(qRound(0.00295 * relSize * (maxx - minx)));
   setPen(pen3);
   QFont fnt = font();
-  fnt.setPointSize(QMAX(qRound(0.0355 * relSize * (maxx - minx)), 1));
+  fnt.setPointSize(QMAX(qRound(0.0355 * fDpi * relSize * (maxx - minx)), 1));
   setFont(fnt);
 #ifndef NO_CLIPPING
-  setClipRect((minx + (dx >> 1)) / pFactor , (maxy + (dx >> 1)) / pFactor,
-              (maxx - minx - dx) / pFactor, (miny - maxy - dx) / pFactor);
+  setClipRect((minx + (_dx >> 1)) / pFactor , (maxy + (_dx >> 1)) / pFactor,
+              (maxx - minx - _dx) / pFactor, (miny - maxy - _dx) / pFactor);
   setClipping(false);
 #endif
 }
@@ -303,163 +314,229 @@ void KGraph::frame()
 }
 
 void KGraph::raster(double xtic, double ytic, int intx, int inty, int mode,
-                    bool xTop, bool yRight)
+                    double xOff, double yOff, bool xBottom, bool xTop,
+                    bool yLeft, bool yRight)
 {
-  if (logx) {
-    if (xtic < 2.5)
-      xtic = 2.0;
-    else
-      if (xtic < 5.5)
-        xtic = 3.0;
-      else
-        if (xtic < 99.0)
-          xtic = 10.0;
-  }
-  if (logy) {
-    if (ytic < 2.5)
-      ytic = 2.0;
-    else
-      if (ytic < 5.5)
-        ytic = 3.0;
-      else
-        ytic = 10.0;
-  }
+  if (mode == Nothing)
+    return;
   double dx = 0.0, dy, x, y;
-  if (mode) {
+  if (mode > FrameOnly) {
+    if (logx) {
+      if (xtic < 2.5)
+        xtic = 2.0;
+      else
+        if (xtic < 5.5)
+          xtic = 3.0;
+        else
+          if (xtic < 99.0)
+            xtic = 10.0;
+    }
+    if (logy) {
+      if (ytic < 2.5)
+        ytic = 2.0;
+      else
+        if (ytic < 5.5)
+          ytic = 3.0;
+        else
+          if (ytic < 99.0)
+            ytic = 10.0;
+    }
     setPen(pen2);
     dx = 0.99999 * (xmax - xmin);
     dy = 0.99999 * (ymax - ymin);
-    if (mode <= 2) {
+    if (mode <= AxesWithLabels) {
       dx *= 0.02;
       dy = -dx * xscal / yscal;
     }
-    for (x = xmin; x < xmax;)
+    if (xOff && (!logx) && mode <= AxesWithLabels && intx > 1) {
+      double d = xtic / intx;
+      for (double xm = xmin + xOff - d; xm > xmin; xm -= d) {
+        if (xBottom)
+          line(xm, ymin, xm, ymin + 0.5 * dy);
+        if (xTop)
+          line(xm, ymax, xm, ymax - 0.5 * dy);
+      }
+    }
+    for (x = xmin + xOff; x < xmax;)
       if (logx) {
         double xa = x;
         double mant = incLog(&x, xtic);
-        if (x < xmax)
+        if (x < xmax) {
+          if (xBottom)
+            line(x, ymin, x, ymin + dy);
           if (xTop)
             line(x, ymax, x, ymax - dy);
-          else
-            line(x, ymin, x, ymin + dy);
+        }
         if (intx > 1) {
           double xd = pow(10.0, xa - mant);
           for (;;) {
             xa = log10(pow(10.0, xa) + xd);
             if ((xa >= x) || (xa >= xmax))
               break;
+            if (xBottom)
+              line(xa, ymin, xa,
+                   ymin + (mode <= AxesWithLabels ? 0.5 * dy : dy));
             if (xTop)
-              line(xa, ymax, xa, ymax - (mode <= 2 ? 0.5 * dy : dy));
-            else
-              line(xa, ymin, xa, ymin + (mode <= 2 ? 0.5 * dy : dy));
+              line(xa, ymax, xa,
+                   ymax - (mode <= AxesWithLabels ? 0.5 * dy : dy));
           }
         }
       } else {
-        if (mode <= 2  && intx > 1) {
+        if (mode <= AxesWithLabels && intx > 1) {
           double xm = x;
           for (int i = 1; i < intx; i++)
-            if ((xm += xtic / intx) < xmax)
+            if ((xm += xtic / intx) < xmax) {
+              if (xBottom)
+                line(xm, ymin, xm, ymin + 0.5 * dy);
               if (xTop)
                 line(xm, ymax, xm, ymax - 0.5 * dy);
-              else
-                line(xm, ymin, xm, ymin + 0.5 * dy);
+            }
         }
-        if ((x += xtic) < xmax)
+        if (x > xmin) {
+          if (xBottom)
+            line(x, ymin, x, ymin + dy);
           if (xTop)
             line(x, ymax, x, ymax - dy);
-          else
-            line(x, ymin, x, ymin + dy);
+        }
+        x += xtic;
       }
-    for (y = ymin; y < ymax;)
+    if (yOff && (!logy) && mode <= AxesWithLabels && inty > 1) {
+      double d = ytic / inty;
+      for (double ym = ymin + yOff - d; ym > ymin; ym -= d) {
+        if (yLeft)
+          line(xmin, ym, xmin + 0.5 * dx, ym);
+        if (yRight)
+          line(xmax, ym, xmax - 0.5 * dx, ym);
+      }
+    }
+    for (y = ymin + yOff; y < ymax;)
       if (logy) {
         double ya = y;
         double mant = incLog(&y, ytic);
-        if (y < ymax)
+        if (y < ymax) {
+          if (yLeft)
+            line(xmin, y, xmin + dx, y);
           if (yRight)
             line(xmax, y, xmax - dx, y);
-          else
-            line(xmin, y, xmin + dx, y);
+        }
         if (inty > 1) {
           double yd = pow(10.0, ya - mant);
           for (;;) {
             ya = log10(pow(10.0, ya) + yd);
             if ((ya >= y) || (ya >= ymax))
               break;
+            if (yLeft)
+              line(xmin, ya,
+                   xmin + (mode <= AxesWithLabels ? 0.5 * dx : dx), ya);
             if (yRight)
-              line(xmax, ya, xmax - (mode <= 2 ? 0.5 * dx : dx), ya);
-            else
-              line(xmin, ya, xmin + (mode <= 2 ? 0.5 * dx : dx), ya);
+              line(xmax, ya,
+                   xmax - (mode <= AxesWithLabels ? 0.5 * dx : dx), ya);
           }
         }
       } else {
-        if (mode <= 2  && intx > 1) {
+        if (mode <= AxesWithLabels && inty > 1) {
           double ym = y;
           for (int i = 1; i < inty; i++)
-            if ((ym += ytic / inty) < ymax)
+            if ((ym += ytic / inty) < ymax) {
+              if (yLeft)
+                line(xmin, ym, xmin + 0.5 * dx, ym);
               if (yRight)
                 line(xmax, ym, xmax - 0.5 * dx, ym);
-              else
-                line(xmin, ym, xmin + 0.5 * dx, ym);
+            }
         }
-        if ((y += ytic) < ymax)
+        if (y > ymin) {
+          if (yLeft)
+            line(xmin, y, xmin + dx, y);
           if (yRight)
             line(xmax, y, xmax - dx, y);
-          else
-            line(xmin, y, xmin + dx, y);
+        }
+        y += ytic;
       }
   }
-  setPen(pen1);
-  if ((mode == 2) || (mode == 4)) {
+  if ((mode == AxesWithLabels) || (mode == GridWithLabels)) {
     QString label;
+    double yB = 0.0, yT = 0.0;
+    if (xBottom)
+      yB = ymin + 0.03225 * relSize * (xmax - xmin) * xscal / yscal;
     if (xTop)
-      y = ymax - 0.03225 * relSize * (xmax - xmin) * xscal / yscal;
-    else
-      y = ymin + 0.03225 * relSize * (xmax - xmin) * xscal / yscal;
+      yT = ymax - 0.03225 * relSize * (xmax - xmin) * xscal / yscal;
     int ix, iy;
     bool e = (logx && (xtic > 9.9) && ((xmax > 2.999999) || (xmin < -2.999999)));
     int nd = ndigx;
-    for (x = xmin; x <= (xmax + 1.0e-3 * dx);) {
+    double xe = xmax + 1.0e-3 * (xmax - xmin);
+    for (x = xmin + xOff; x <= xe;) {
       if (x > xmax)
          x = xmax;
-      r2i(x, y, &ix, &iy);
-      if (e)
-        text(label.sprintf("10#nu%d", qRound(x)), ix, iy, 2);
-      else {
+      if (e) {
+        if (xBottom) {
+          r2i(x, yB, &ix, &iy);
+          text(label.sprintf("10#nu%d", qRound(x)), ix, iy, 2);
+        }
+        if (xTop) {
+          r2i(x, yT, &ix, &iy);
+          text(label.sprintf("10#nu%d", qRound(x)), ix, iy, 2);
+        }
+      } else {
         if (logx)
           if (x < 0)
             nd = 1 - int(x + 0.01);
           else
             nd = -1;
         int l = prenum(logx ? pow(10.0, x) : x, nd, label);
-        drawText(ix, iy, 0, 0, AlignCenter | DontClip, label, l);
+        setPen(pen1);
+        if (xBottom) {
+          r2i(x, yB, &ix, &iy);
+          drawText(ix, iy, 0, 0, AlignCenter | DontClip, label, l);
+        }
+        if (xTop) {
+          r2i(x, yT, &ix, &iy);
+          drawText(ix, iy, 0, 0, AlignCenter | DontClip, label, l);
+        }
       }
       if (logx)
         incLog(&x, xtic);
       else
         x += xtic;
     }
+    double xL = 0.0, xR = 0.0;
+    if (yLeft)
+      xL = xmin - 0.025 * relSize * (xmax - xmin);
     if (yRight)
-      x = xmax + 0.025 * relSize * (xmax - xmin);
-    else
-      x = xmin - 0.025 * relSize * (xmax - xmin);
+      xR = xmax + 0.025 * relSize * (xmax - xmin);
     dy = -0.0125 * relSize * (xmax - xmin) * xscal / yscal;
     e = (logy && (ytic > 9.9) && ((ymax > 2.999999) || (ymin < -2.999999)));
     nd = ndigy;
-    for (y = ymin; y <= (ymax + 1.0e-3 * dy);) {
+    double ye = ymax + 1.0e-3 * (ymax - ymin);
+    for (y = ymin + yOff; y <= ye;) {
       if (y > ymax)
         y = ymax;
-      r2i(x, y, &ix, &iy);
-      if (e)
-        text(label.sprintf("10#nu%d", qRound(y)), ix, iy, yRight ? 1 : 3);
-      else {
+      if (e) {
+        if (yLeft) {
+          r2i(xL, y, &ix, &iy);
+          text(label.sprintf("10#nu%d", qRound(y)), ix, iy, 3);
+        }
+        if (yRight) {
+          r2i(xR, y, &ix, &iy);
+          text(label.sprintf("10#nu%d", qRound(y)), ix, iy, 1);
+        }
+      } else {
         if (logy)
           if (y < 0)
             nd = 1 - int(y + 0.01);
           else
             nd = -1;
         int l = prenum(logy ? pow(10.0, y) : y, nd, label);
-        drawText(ix, iy - 1, 0, 0, (yRight ? AlignLeft : AlignRight) |
-                 AlignVCenter | DontClip, label, l);
+        setPen(pen1);
+        if (yLeft) {
+          r2i(xL, y, &ix, &iy);
+          drawText(ix, iy - 1, 0, 0, AlignRight | AlignVCenter | DontClip,
+                   label, l);
+        }
+        if (yRight) {
+          r2i(xR, y, &ix, &iy);
+          drawText(ix, iy - 1, 0, 0, AlignLeft | AlignVCenter | DontClip,
+                   label, l);
+        }
       }
       if (logy)
         incLog(&y, ytic);
@@ -467,25 +544,34 @@ void KGraph::raster(double xtic, double ytic, int intx, int inty, int mode,
         y += ytic;
     }
   }
+  setPen(pen1);
   frame();
   setPen(pen3);
 }
 
-void KGraph::setDir(double ang)
+void KGraph::setDir(double dir)
 {
-  textdir = ang;
+  textdir = dir;
+  if (textdir) {
+    relang = 0.01745329252 * textdir;
+    relsin = sin(relang);
+    relcos = cos(relang);
+  } else {
+    relang = relsin = 0.0;
+    relcos = 1.0;
+  }
 }
 
-void KGraph::text(const char* str, int ix, int iy, int iorg, QRect* bRect)
+void KGraph::text(const QString& str, int ix, int iy, int iorg, QRect* bRect)
 {
-  if (int l = strlen(str)) {
+  if (int l = str.length()) {
     QWMatrix wm = worldMatrix();
     translate(double(ix), double(iy));
     if (textdir)
       rotate(-textdir);
     setPen(pen1);
     int tf = 0;
-    if (!strchr(str, '#')) {
+    if (!str.contains('#')) {
       switch (iorg) {
         case 1:
           tf = AlignLeft | AlignVCenter | DontClip;
@@ -496,7 +582,7 @@ void KGraph::text(const char* str, int ix, int iy, int iorg, QRect* bRect)
         case 3:
           tf = AlignRight | AlignVCenter | DontClip;
       }
-      drawText(0, 0, 0, 0, tf, str, l, bRect);
+      drawText(0, 0, 0, 0, tf, str, -1, bRect);
     } else {
       const char fontcode[] = "NHBOSZI", levcode[] = "NUD";
       const int iface[7] = {0, 0, 0, 0, 1, 2, 0};
@@ -519,14 +605,14 @@ void KGraph::text(const char* str, int ix, int iy, int iorg, QRect* bRect)
           if (iorg != 1)
             ix2 = -idx;
         }
-        const char* s = str;
+        const char* s = str.latin1();
         int ilev = 0;
         while (*s) {
           if ((l = ((c = strchr(s, '#'))) ? (int) (c - s) : strlen(s))) {
             QRect r;
             if (j == 0) {
               r = boundingRect(0, 0, maxx, maxy,
-                                     AlignLeft | AlignVCenter, s, l);
+                               AlignLeft | AlignVCenter, s, l);
               idx += r.width();
             } else {
               idy = (ilev) ? -qRound(dr * ys[ilev - 1]) : 0;
@@ -552,14 +638,20 @@ void KGraph::text(const char* str, int ix, int iy, int iorg, QRect* bRect)
             if (!(f = strchr(levcode, code)))
               break;
             setFont(QFont(face[iface[ifnt]],
-                    QMAX(qRound(((ilev = (int) (f - levcode)) ? 0.0251 : 0.0355)
-                         * relSize * (maxx - minx)), 1),
-                    weight[ifnt], italic[ifnt], QFont::AnyCharSet));
+                          QMAX(qRound(((ilev = (int) (f - levcode)) ?
+                                       0.0251 : 0.0355) *
+                                       relSize * fDpi * (maxx - minx)), 1),
+#if (KDE_VERSION_MAJOR > 2) || (KDE_VERSION_MINOR > 8)
+                          weight[ifnt], italic[ifnt]));
+#else
+                          weight[ifnt], italic[ifnt],
+                          iface[ifnt] ? QFont::AnyCharSet : QFont::Latin1));
+#endif
           } else
             s += l;
         }
         setFont(QFont(face[iface[0]],
-                QMAX(qRound(0.0355 * relSize * (maxx - minx)), 1),
+                QMAX(qRound(0.0355 * relSize * fDpi * (maxx - minx)), 1),
                 weight[0], italic[0]));
       }
     }
@@ -574,7 +666,8 @@ void KGraph::textcm(const QString& str, double x, double y, int al,
                     QRect* bRect)
 {
   double xa, ya;
-  cm2r(x, y + 0.18, 1.0, 1.0, &xa, &ya);
+  double dh = 0.013 * relSize * (maxx - minx) / (scalx * factor);
+  cm2r(x - dh * relsin, y + dh * relcos, 1.0, 1.0, &xa, &ya);
   int ix, iy;
   r2i(logx ? log10(xa) : xa, logy ? log10(ya) : ya, &ix, &iy);
   text(str, ix, iy, al, bRect);
@@ -598,8 +691,8 @@ void KGraph::arrow(double x, double y, double dir, double len,
   drawPolygon(a);
   setBrush(NoBrush);
   setPen(pen1);
-  int idx2 = QMIN(idx, -qRound(factor * scalx * len));
-  if (len)
+  int idx2 = -qRound(factor * scalx * len);
+  if (idx2 <= idx)
     drawLine(idx, 0, idx2, 0);
   if (bRect)
     *bRect = xForm(QRect(idx2, -idy, -idx2, idy + idy));
@@ -607,10 +700,33 @@ void KGraph::arrow(double x, double y, double dir, double len,
   setWorldMatrix(wm);
 }
 
-void KGraph::rectangle(double x, double y, double w, double h,
-                       bool filled, QRect* bRect)
+void KGraph::arc(double x, double y, double w, double h, double a,
+                 double alen, double dir, QRect* bRect)
 {
-  QRect r(fromcm(x, y + h), fromcm(x + w, y));
+  QWMatrix wm = worldMatrix();
+  QPoint p = fromcm(x, y);
+  translate(double(p.x()), double(p.y()));
+  if (dir)
+    rotate(-dir);
+  int iw = qRound(factor * scalx * w);
+  int ih = -qRound(factor * scaly * h);
+  QRect r(-iw / 2, -ih / 2, iw, ih);
+  drawArc(r, qRound(16 * a), qRound(16 * alen));
+  if (bRect)
+    *bRect = xForm(r);
+  setWorldMatrix(wm);
+}
+
+void KGraph::rectangle(double x, double y, double w, double h,
+                       bool filled, double dir, QRect* bRect)
+{
+  QWMatrix wm = worldMatrix();
+  QPoint p = fromcm(x, y);
+  translate(double(p.x()), double(p.y()));
+  if (dir)
+    rotate(-dir);
+  int idy = qRound(factor * scaly * h);
+  QRect r(0, idy, qRound(factor * scalx * w), -idy);
   if (filled) {
     setBrush(pen3.color());
     setPen(NoPen);
@@ -621,15 +737,21 @@ void KGraph::rectangle(double x, double y, double w, double h,
     setPen(pen3);
   }
   if (bRect)
-    *bRect = r;
+    *bRect = xForm(r);
+  setWorldMatrix(wm);
 }
 
 void KGraph::ellipse(double x, double y, double w, double h,
-                     bool filled, QRect* bRect)
+                     bool filled, double dir, QRect* bRect)
 {
-  double dx = 0.5 * w;
-  double dy = 0.5 * h;
-  QRect r(fromcm(x - dx , y + dy), fromcm(x + dx, y - dy));
+  QWMatrix wm = worldMatrix();
+  QPoint p = fromcm(x, y);
+  translate(double(p.x()), double(p.y()));
+  if (dir)
+    rotate(-dir);
+  int iw = qRound(factor * scalx * w);
+  int ih = -qRound(factor * scaly * h);
+  QRect r(-iw / 2, -ih / 2, iw, ih);
   if (filled) {
     setBrush(pen3.color());
     setPen(NoPen);
@@ -640,7 +762,8 @@ void KGraph::ellipse(double x, double y, double w, double h,
     setPen(pen3);
   }
   if (bRect)
-    *bRect = r;
+    *bRect = xForm(r);
+  setWorldMatrix(wm);
 }
 
 void KGraph::letH(const QString& str, bool xTop)
@@ -690,6 +813,26 @@ void KGraph::arcrel(double x, double y, double r, double angst, double angend)
           qRound(16.0 * angst), qRound(16.0 * angend));
 }
 
+double KGraph::incLog(double* x, double f)
+{
+  double mant = *x - int(*x + 1.0e-6 * fabs(*x));
+  if (fabs(mant) < 1.0e-9)
+    mant = 0.0;
+  if (mant < 0.0)
+    mant += 1.0;
+  if (mant < 0.1)
+    *x += log10(f);
+  else
+    if (mant < 0.4)
+      *x += log10(2.5);
+    else
+      if (mant < 0.6)
+        *x += log10(10.0 / 3.0);
+      else
+        *x += log10(2.0);
+  return mant;
+}
+
 void KGraph::plArray(const double *x, const double *y, double fx, double fy,
                      int n, bool clip)
 {
@@ -737,8 +880,8 @@ void KGraph::plArray(const double *x, const double *y, double fx, double fy,
                         2, 0, 0, 2, -2},
               as[10] = {0, 5, 10, 14, 19, 28, 33, 38, 43, 48};
     int j, l = 0, r;
+    double f = 0.005 * relSize * (maxx - minx);
     for (int i = 0; i < n; i++) {
-      double f = 0.005 * relSize * (maxx - minx);
       xa = fx * x[i];
       ya = fy * y[i];
       if (((!logx) || (xa > 0)) && ((!logy) || (ya > 0))) {
@@ -819,6 +962,11 @@ void KGraph::plError(const double* x, const double* y, const double* e,
 #endif
 }
 
+void KGraph::resetDash()
+{
+  dcur = 0.0;
+}
+
 void KGraph::linetype(int is)
 {
   const int pat[9][2] = {{8, 8}, {7, 0}, {2, 0}, {1, 0}, {9, 0},
@@ -839,6 +987,7 @@ void KGraph::letxy(const QString& str, int iex, int iax, bool opposite)
     double x, y;
     QString label;
     double dx = 0.025 * relSize * (xmax - xmin);
+    double dirsav = textdir;
     if (iax == 2) {
       int l = 3;
       if (logy)
@@ -853,7 +1002,7 @@ void KGraph::letxy(const QString& str, int iex, int iax, bool opposite)
       else
         x = xmin - (0.75 * l + 0.71) * dx;
       y = 0.5 * (ymin + ymax);
-      textdir = (opposite ? -90.0 : 90.0);
+      setDir(opposite ? -90.0 : 90.0);
     } else {
       x = 0.5 * (xmin + xmax);
       if (opposite)
@@ -868,34 +1017,6 @@ void KGraph::letxy(const QString& str, int iex, int iax, bool opposite)
     else
       text(str, ix, iy, 2);
     if (iax == 2)
-      textdir = 0.0;
+      setDir(dirsav);
   }
 }
-
-int KGraph::prenum(double a, int ndig, QString& label)
-{
-  label.sprintf("%.*f", QMAX(0, ndig), a);
-  if (ndig == 0)
-    label += ".";
-  return label.length();
-}
-
-double KGraph::incLog(double* x, double f)
-{
-  double mant = *x - int(*x + 1.0e-6 * fabs(*x));
-  if (fabs(mant) < 1.0e-9)
-    mant = 0.0;
-  if (mant < 0.0)
-    mant += 1.0;
-  if (mant < 0.1)
-    *x += log10(f);
-  else
-    if (mant < 0.4)
-      *x += log10(2.5);
-    else
-      if (mant < 0.6)
-        *x += log10(10.0 / 3.0);
-      else
-        *x += log10(2.0);
-  return mant;
-}
