@@ -7,7 +7,7 @@
 
     begin                : Sat Apr 24 15:14:00 MEST 1999
 
-    copyright            : (C) 2001 by Werner Stille
+    copyright            : (C) 2002 by Werner Stille
     email                : stille@uni-freiburg.de
  ***************************************************************************/
 
@@ -30,17 +30,15 @@
 #include <dcopobject.h>
 #include <kmainwindow.h>
 #include <kurl.h>
-#include "kpldoc.h"
 
 class QStrList;
 class KURL;
 class KAccel;
 class KAction;
-class KListAction;
 class KToggleAction;
 class KRecentFilesAction;
-class KplDoc;
 class KplView;
+class KplDoc;
 class ItemDlg;
 class ArrayItem;
 class FunItem;
@@ -50,61 +48,59 @@ class FunItem;
   * Provides the kpl top level main window with menubar, toolbar and statusbar.
   * @author Werner Stille
   */
-
 class KplApp : public KMainWindow, public DCOPObject
 {
   K_DCOP
   Q_OBJECT
-
 public:
   /** Constructor. Creates a kpl window. */
   KplApp(QWidget* parent = 0, const char* name = 0);
   /** Destructor. */
   ~KplApp();
-	/**
-	 * Returns a pointer to the current document connected to the KMainWindow
-	 * instance and is used by the View class to access the document object's
-	 * methods.
+  /**
+   * Returns a pointer to the current document connected to the KMainWindow
+   * instance and is used by the View class to access the document object's
+   * methods.
    * @return pointer to the current document.
-   */	
-	KplDoc* getDocument() const; 	
+   */
+  KplDoc* getDocument() const; 	
   /**
    * Opens files from URL list.
    * @param list pointer to list with URLs.
    * @param type one of the following: Unknown, Data, Plot.
    */
-  void openList(KURL::List* list, KplDoc::FileTypes type = KplDoc::Unknown);
+  void openList(KURL::List* list, int type = 0);
 
 k_dcop:
   /** Creates a new plot in the current view. */
   void newPlot();
-	/**
-	 * Opens a plot file.
+  /**
+   * Opens a plot file.
    * @param url URL of the file.
    * @return true when when succeeded.
-	 */
-  bool openPlot(const QString& url);
-	/**
-	 * Opens a data file.
+   */
+  int openPlot(const QString& url);
+  /**
+   * Opens a data file.
    * @param url URL of the file.
    * @return true when when succeeded.
-	 */
-  bool openData(const QString& url);
-	/** Performs plot. */
+   */
+  int openData(const QString& url);
+  /** Performs plot. */
   void doPlot();
-	/**
-	 * Saves a plot file.
+  /**
+   * Saves a plot file.
    * @param url URL of the file.
    * @return true when when succeeded.
-	 */
-  bool savePlot(const QString& url);
+   */
+  int savePlot(const QString& url);
   /**
    * Generates PostScript file.
    * @param url URL of the file.
    * @param landscape true for landscape orientation.
    * @return true when when succeeded.
    */
-  bool savePS(const QString& url, int landscape);
+  int savePS(const QString& url, int landscape);
   /**
    * Prints plot.
    * @param printerName name of the printer.
@@ -113,8 +109,8 @@ k_dcop:
    * @param numCopies number of copies.
    * @return true when when succeeded.
    */
-  bool printPlot(const QString& printerName, const QString & fileName,
-                 int landscape, int numCopies);
+  int printPlot(const QString& printerName, const QString & fileName,
+                int landscape, int numCopies);
   /**
    * Sets autoplot option.
    * @param setting true to plot automatically after changes.
@@ -212,28 +208,28 @@ k_dcop:
    * @param url URL of the file.
    * @return true when when succeeded.
    */
-  bool loadAutofitPar(const QString& url);
+  int loadAutofitPar(const QString& url);
   /**
    * Sets Autofit function parameter value.
    * @param iPar parameter index.
    * @param value parameter value.
    * @return true when when succeeded.
    */
-  bool setAutofitPar(int iPar, double value);
+  int setAutofitPar(int iPar, double value);
   /**
    * Enables/disables (Auto)fit function parameter.
    * @param iPar parameter index.
    * @param fit true enables parameter fitting.
    * @return true when when succeeded.
    */
-  bool enableFitPar(int iPar, int fit);
+  int enableFitPar(int iPar, int fit);
   /**
    * Loads error model function parameters from file.
    * @param url URL of the file.
    * @param iArr array list index.
    * @return true when when succeeded.
    */
-  bool loadErrModPar(const QString& url, int iArr);
+  int loadErrModPar(const QString& url, int iArr);
   /**
    * Sets error model function parameter value.
    * @param iPar parameter index.
@@ -241,20 +237,27 @@ k_dcop:
    * @param iArr array list index.
    * @return true when when succeeded.
    */
-  bool setErrModPar(int iPar, double value, int iArr);
+  int setErrModPar(int iPar, double value, int iArr);
   /**
    * Deletes plot item.
    * @param iItem item index.
    * @return true when when succeeded.
    */
-  bool deleteItem(int iItem);
+  int deleteItem(int iItem);
   /**
    * Moves plot item.
    * @param is source item index.
    * @param id destination item index.
    * @return true when when succeeded.
    */
-  bool moveItem(int is, int id);
+  int moveItem(int is, int id);
+  /**
+   * Activates or deactivates plot item.
+   * @param iItem item index.
+   * @param active true for activating item (making visible).
+   * @return true when when succeeded.
+   */
+  int activateItem(int iItem, int active);
   /**
    * Appends new frame item corresponding to arguments.
    * @param x0 left page margin in centimeters.
@@ -285,7 +288,7 @@ k_dcop:
    * @param relSize relative size of text, markers and lines.
    * @param active true for visible item.
    */
-	void newFrameItem(double x0, double w, double y0, double h,
+  void newFrameItem(double x0, double w, double y0, double h,
                     double xmin, double xmax, double ymin, double ymax,
                     int logx, int logy, int ndigx, int ndigy,
                     const QString& colFrame, const QString& colGrid,
@@ -312,9 +315,9 @@ k_dcop:
    * @param active true for visible item.
    * @return true when when succeeded.
    */
-	bool newArrayItem(const QString& path, int ix, int iy, int ie, int istart,
-                    int n, int errbars, int symb, const QString& color,
-                    double fx, double fy, int active);
+  int newArrayItem(const QString& path, int ix, int iy, int ie, int istart,
+                   int n, int errbars, int symb, const QString& color,
+                   double fx, double fy, int active);
   /**
    * Appends new function item.
    * @param path absolute path of the function library.
@@ -330,9 +333,9 @@ k_dcop:
    * @param active true for visible item.
    * @return true when when succeeded.
    */
-	bool newFunItem(const QString& path, const QString& name,
-                  double xmin, double xmax, double dx,
-                  int symb, const QString& color,
+  int newFunItem(const QString& path, const QString& name,
+                 double xmin, double xmax, double dx,
+                 int symb, const QString& color,
                   double fx, double fy, int active);
   /**
    * Appends new parametric function item.
@@ -351,11 +354,182 @@ k_dcop:
    * @param active true for visible item.
    * @return true when when succeeded.
    */
-	bool newParFunItem(const QString& pathx, const QString& namex,
-	                   const QString& pathy, const QString& namey,
-                     double tmin, double tmax, double dt,
-                     int symb, const QString& color,
-                     double fx, double fy, int active);
+  int newParFunItem(const QString& pathx, const QString& namex,
+                    const QString& pathy, const QString& namey,
+                    double tmin, double tmax, double dt,
+                    int symb, const QString& color,
+                    double fx, double fy, int active);
+  /**
+   * Appends new spline item.
+   * @param deriv order of derivative.
+   * @param low lower limit for spline integral.
+   * @param xmin minimum argument value.
+   * @param xmax maximum argument value.
+   * @param dx argument stepsize.
+   * @param symb 0 for circles, < 0 for different markers,
+   * >0 for different line types.
+   * @param color color for function plot.
+   * @param relSize relative size of lines or symbols.
+   * @param fx normalization factor for x values.
+   * @param fy normalization factor for y values.
+   * @param active true for visible item.
+   */
+  void newSplineItem(int deriv, double low, double xmin, double xmax,
+                     double dx, int symb, const QString& color,
+                     double relSize, double fx, double fy, int active);
+  /**
+   * Appends new 3D array item.
+   * @param x0 left page margin in centimeters.
+   * @param w width in centimeters.
+   * @param y0 bottom page margin in centimeters.
+   * @param h height in centimeters.
+   * @param xmin minimum x value.
+   * @param xmax maximum x value.
+   * @param ymin minimum y value.
+   * @param ymax maximum y value.
+   * @param zmin minimum z value.
+   * @param zmax maximum z value.
+   * @param logx true for logarithmic x axis.
+   * @param logy true for logarithmic y axis.
+   * @param logz true for logarithmic z axis.
+   * @param ndigx number of digits for x axis.
+   * @param ndigy number of digits for y axis.
+   * @param ndigz number of digits for z axis.
+   * @param colFrame color for frame and labels.
+   * @param colGrid color for grids and tics.
+   * @param colData color for data.
+   * @param xtic distance between major tics of x axis.
+   * @param ytic distance between major tics of y axis.
+   * @param ztic distance between major tics of z axis.
+   * @param mticx number of intervals between major tics of x axis.
+   * @param mticy number of intervals between major tics of y axis.
+   * @param mticz number of intervals between major tics of z axis.
+   * @param frame true to show frame.
+   * @param gridmode 0 for frame only, 1 for axes with tics,
+   * 2 for axes with tics and labels.
+   * @param gridmode3 0 for lines parallel to x, 1 for lines parallel to z,
+   * 2 for x,z grid.
+   * @param xBox x box width in cm.
+   * @param yBox y box width in cm.
+   * @param zBox z box width in cm.
+   * @param xRef x position of center of box bottom plane.
+   * @param yRef y position of center of box bottom plane.
+   * @param phi angle of rotation about y axis in deg.
+   * @param theta tilt angle of y axis in deg.
+   * @param iex logarithm of normalization factor for x axis.
+   * @param iey logarithm of normalization factor for y axis.
+   * @param iez logarithm of normalization factor for z axis.
+   * @param sx x axis text.
+   * @param sy y axis text.
+   * @param sz z axis text.
+   * @param sh heading text.
+   * @param relSize relative size of text, markers and lines.
+   * @param path absolute path of the data file.
+   * @param ix x column index.
+   * @param iy y column index.
+   * @param iz z column index.
+   * @param ie y error column index.
+   * @param istart start row index.
+   * @param n number of data points.
+   * @param errbars true to plot error bars.
+   * @param smf smoothing factor.
+   * @param dx x argument stepsize.
+   * @param dz z argument stepsize.
+   * @param fx normalization factor for x values.
+   * @param fy normalization factor for y values.
+   * @param fz normalization factor for z values.
+   * @param active true for visible item.
+   * @return true when when succeeded.
+   */
+  int newArray3DItem(double x0, double w, double y0, double h,
+                     double xmin, double xmax, double ymin, double ymax,
+                     double zmin, double zmax, int logx, int logy,
+                     int logz, int ndigx, int ndigy, int ndigz,
+                     const QString& colFrame, const QString& colGrid,
+                     const QString& colData, double xtic, double ytic,
+                     double ztic, int mticx, int mticy, int mticz,
+                     int frame, int gridmode, int gridmode3D,
+                     double xBox, double yBox, double zBox, double xRef,
+                     double yRef, double phi, double theta, int iex,
+                     int iey, int iez, const QString& sx,
+                     const QString& sy, const QString& sz,
+                     const QString& sh, double relSize,
+                     const QString& path, int ix, int iy, int iz, int ie,
+                     int istart, int n, int errbars,  double smf, double dx,
+                     double dz, double fx, double fy, double fz,
+                     int active);
+  /**
+   * Appends new 3D function item.
+   * @param x0 left page margin in centimeters.
+   * @param w width in centimeters.
+   * @param y0 bottom page margin in centimeters.
+   * @param h height in centimeters.
+   * @param xmin minimum x value.
+   * @param xmax maximum x value.
+   * @param ymin minimum y value.
+   * @param ymax maximum y value.
+   * @param zmin minimum z value.
+   * @param zmax maximum z value.
+   * @param logx true for logarithmic x axis.
+   * @param logy true for logarithmic y axis.
+   * @param logz true for logarithmic z axis.
+   * @param ndigx number of digits for x axis.
+   * @param ndigy number of digits for y axis.
+   * @param ndigz number of digits for z axis.
+   * @param colFrame color for frame and labels.
+   * @param colGrid color for grids and tics.
+   * @param colData color for data.
+   * @param xtic distance between major tics of x axis.
+   * @param ytic distance between major tics of y axis.
+   * @param ztic distance between major tics of z axis.
+   * @param mticx number of intervals between major tics of x axis.
+   * @param mticy number of intervals between major tics of y axis.
+   * @param mticz number of intervals between major tics of z axis.
+   * @param frame true to show frame.
+   * @param gridmode 0 for frame only, 1 for axes with tics,
+   * 2 for axes with tics and labels.
+   * @param gridmode3 0 for lines parallel to x, 1 for lines parallel to z,
+   * 2 for x,z grid.
+   * @param xBox x box width in cm.
+   * @param yBox y box width in cm.
+   * @param zBox z box width in cm.
+   * @param xRef x position of center of box bottom plane.
+   * @param yRef y position of center of box bottom plane.
+   * @param phi angle of rotation about y axis in deg.
+   * @param theta tilt angle of y axis in deg.
+   * @param iex logarithm of normalization factor for x axis.
+   * @param iey logarithm of normalization factor for y axis.
+   * @param iez logarithm of normalization factor for z axis.
+   * @param sx x axis text.
+   * @param sy y axis text.
+   * @param sz z axis text.
+   * @param sh heading text.
+   * @param relSize relative size of text, markers and lines.
+   * @param path absolute path of the function library.
+   * @param name function name.
+   * @param dx x argument stepsize.
+   * @param dz z argument stepsize.
+   * @param fx normalization factor for x values.
+   * @param fy normalization factor for y values.
+   * @param fz normalization factor for z values.
+   * @param active true for visible item.
+   * @return true when when succeeded.
+   */
+  int newFun3DItem(double x0, double w, double y0, double h,
+                   double xmin, double xmax, double ymin, double ymax,
+                   double zmin, double zmax, int logx, int logy,
+                   int logz, int ndigx, int ndigy, int ndigz,
+                   const QString& colFrame, const QString& colGrid,
+                   const QString& colData, double xtic, double ytic,
+                   double ztic, int mticx, int mticy, int mticz,
+                   int frame, int gridmode, int gridmode3D, double xBox,
+                   double yBox, double zBox, double xRef, double yRef,
+                   double phi, double theta, int iex, int iey, int iez,
+                   const QString& sx, const QString& sy,
+                   const QString& sz, const QString& sh, double relSize,
+                   const QString& path, const QString& _name,
+                   double dx, double dz, double fx, double fy,
+                   double fz, int active);
   /**
    * Appends new legend item.
    * @param symb 0 for circles, < 0 for different markers,
@@ -370,7 +544,7 @@ k_dcop:
    * @param relSize relative size of text.
    * @param active true for visible item.
    */
-	void newLegendItem(int symb, const QString& colSymb, double x, double y,
+  void newLegendItem(int symb, const QString& colSymb, double x, double y,
                      const QString& text, const QString& colText,
                      double xoff, double yoff, double relSize,
                      int active);
@@ -386,7 +560,7 @@ k_dcop:
    * @param relSize relative line width.
    * @param active true for visible item.
    */
-	void newTextItem(const QString& text, const QString& color, int align,
+  void newTextItem(const QString& text, const QString& color, int align,
                    double x, double y, double dir, double relSize,
                    int active);
   /**
@@ -401,7 +575,7 @@ k_dcop:
    * @param relSize relative line width.
    * @param active true for visible item.
    */
-	void newLineItem(int symb, const QString& color, double x, double y,
+  void newLineItem(int symb, const QString& color, double x, double y,
                    double dir, double len, double relSize, int active);
   /**
    * Appends new arrow item.
@@ -414,8 +588,24 @@ k_dcop:
    * @param relSize relative size of head.
    * @param active true for visible item.
    */
-	void newArrowItem(const QString& color, double x, double y,
+  void newArrowItem(const QString& color, double x, double y,
                     double dir, double len, double relSize, int active);
+  /**
+    * Constructor. Initializes corresponding to arguments.
+    * @param color color for arc.
+    * @param x x position in cm.
+    * @param y y position in cm.
+    * @param w width in cm.
+    * @param h height in cm.
+    * @param a start angle in degree.
+    * @param alen length in degree.
+    * @param relSize relative line width.
+    * @param ang rotation angle in degree.
+    * @param active true for visible item.
+    */
+  void newArcItem(const QString& color, double x, double y, double w,
+                  double h, double a, double alen, double relSize, double ang,
+                  int active);
   /**
    * Appends new rectangle item.
    * Constructor. Initializes corresponding to arguments.
@@ -428,7 +618,7 @@ k_dcop:
    * @param fill true for filled with color.
    * @param active true for visible item.
    */
-	void newRectangleItem(const QString& color, double x, double y,
+  void newRectangleItem(const QString& color, double x, double y,
                         double w, double h, double relSize, int fill,
                         int active);
   /**
@@ -443,7 +633,7 @@ k_dcop:
    * @param fill true for filled with color.
    * @param active true for visible item.
    */
-	void newEllipseItem(const QString& color, double x, double y,
+  void newEllipseItem(const QString& color, double x, double y,
                       double w, double h, double relSize, int fill,
                       int active);
   /**
@@ -452,7 +642,7 @@ k_dcop:
    * @param autoNorm true for automatic normalization.
    * @return true when when succeeded.
    */
-  bool autoScaleFrame(int iItem, int autoNorm);
+  int autoScaleFrame(int iItem, int autoNorm);
   /**
    * Loads function parameter file.
    * @param url URL of the file.
@@ -461,7 +651,7 @@ k_dcop:
    *  (only for parametric functions).
    * @return true when when succeeded.
    */
-  bool loadPar(const QString& url, int iItem, int yFun);
+  int loadPar(const QString& url, int iItem, int yFun);
   /**
    * Sets function parameter value.
    * @param iItem item index.
@@ -471,7 +661,7 @@ k_dcop:
    *  (only for parametric functions).
    * @return true when when succeeded.
    */
-  bool setPar(int iItem, int iPar, double value, int yFun);
+  int setPar(int iItem, int iPar, double value, int yFun);
   /**
    * Sets array row range.
    * @param iItem item index.
@@ -479,7 +669,7 @@ k_dcop:
    * @param n number of data points.
    * @return true when when succeeded.
    */
-  bool setArrayRange(int iItem, int iStart, int n);
+  int setArrayRange(int iItem, int iStart, int n);
   /**
    * Adds array and function items to item lists for (multidimensional)
    * parameter fitting.
@@ -493,15 +683,33 @@ k_dcop:
    * @param errModName error model function name.
    * @return true when when succeeded.
    */
-  bool addFitItems(int erase, int iArr, int iFun, int errCol,
-                   const QString& errModPath, const QString& errModName);
+  int addFitItems(int erase, int iArr, int iFun, int errCol,
+                  const QString& errModPath, const QString& errModName);
   /**
    * Performs parameter fit.
    * @param nonLin true for nonlinear parameter fits.
    * @param savePar true to save resulting parameter files.
    * @param follow true to follow parameter sets in fit series.
    */
-  bool fit(int nonLin, int savePar, int follow);
+  int fit(int nonLin, int savePar, int follow);
+  /**
+   * Performs smoothing spline fit.
+   * @param iArr array item index.
+   * @param iSpl spline item index.
+   * @param deg degree of spline.
+   * @param smf smoothing factor.
+   * @param xmin lower boundary of approximation interval.
+   * @param xmax upper boundary of approximation interval.
+   * @param errCol true for data weighing by error column, false for weighing
+   * by error model function.
+   * @param errModPath absolute path of the library for the error model
+   * function.
+   * @param errModName error model function name.
+   * @return true when when succeeded.
+   */
+  int splineFit(int iArr, int iSpl, int deg, double smf, double xmin,
+                double xmax, int errCol, const QString& errModPath,
+                const QString& errModName);
   /**
    * Returns chi-square of last fit.
    * @return chi-square.
@@ -512,14 +720,14 @@ k_dcop:
    * @param url URL of the file.
    * @return true when when succeeded.
    */
-  bool saveFitPar(const QString& url);
+  int saveFitPar(const QString& url);
   /**
    * Exports function value table to file.
    * @param url URL of the file.
    * @param iItem item index.
    * @return true when when succeeded.
    */
-  bool exportValues(const QString& url, int iItem);
+  int exportValues(const QString& url, int iItem);
 
 protected:
   /** Initializes menu and toolbar. */
@@ -530,16 +738,16 @@ protected:
   void initStatusBar();
   /** Creates and initialize new statusbar. */
   void newStatusBar();
-	/**
+  /**
    * initKeyAccel creates the keyboard accelerator items for the available slots and changes the menu accelerators.
    * @see KAccel
    */
-	void initKeyAccel();
-	/**
+  void initKeyAccel();
+  /**
    * Initializes the document object of the main window that is connected to the view in initView().
    * @see initView();
    */
-	void initDocument();
+  void initDocument();
   /**
    * Creates the centerwidget of the KMainWindow instance and sets it as the view.
    */
@@ -555,7 +763,7 @@ protected:
    * Opens an existing URL or local file for reading.
    * @param type one of the following: Unknown, Data, Plot.
    */
-  void openRead(KplDoc::FileTypes type);
+  void openRead(int type);
   /**
    * Saves plot file using current or new name.
    * @param current true for save under current name.
@@ -601,10 +809,11 @@ protected:
   KAction* fileClose;
   KAction* filePrint;
   KAction* filePlot;
+  KAction* filePSP;
+  KAction* filePSL;
   KAction* editUndo;
   KAction* editRedo;
   KAction* viewReload;
-  KListAction* filePS;
   KToggleAction* showToolbar;
   KToggleAction* showStatusbar;
   KToggleAction* autoplot;
@@ -632,10 +841,10 @@ protected:
   friend KplView;
 
 public slots:
-	/**
-	 * Opens a file.
+  /**
+   * Opens a file.
    * @param url URL of the file.
-	 */
+   */
   void openDocumentURL(const KURL& url);
   /**
    * Opens files from URL list.
@@ -660,11 +869,15 @@ protected slots:
   void slotFilePlot();
   /** Plots the current file conditionally. */
   void slotPlotCond();
+  /** Generates PostScript file (portrait orientation). */
+  void slotFilePSP();
+  /** Generates PostScript file (landscape orientation). */
+  void slotFilePSL();
   /**
    * Generates PostScript file.
-   * @param landscape != 0 for landscape orientation.
+   * @param landscape true for landscape orientation.
    */
-  void slotFilePS(int landscape);
+  void slotFilePS(bool landscape);
   /** Prints the current file. */
   void slotFilePrint();
   /** Opens a new kpl window. */
@@ -713,6 +926,8 @@ protected slots:
   void slotOptionsAutoFit();
   /** Saves options. */
   void slotOptionsSave();
+  /** Displays the help page. */
+  void slotHelpContents();
   /**
    * Changes the status message.
    * @param text pointer to new message text.
@@ -733,8 +948,9 @@ protected slots:
   /**
    * Updates mouse position on statusbar.
    * @param pos position of the mouse cursor.
+   * @param showValues true to show Values in data units.
    */
-  void updStatus(const QPoint& pos);
+  void updStatus(const QPoint& pos, bool showValues);
   /** Clears mouse position information on statusbar. */
   void clearStatus();
   /** Shrinks window. */
