@@ -12,7 +12,6 @@
 #include <kdatastream.h>
 
 
-
 KplApp_stub::KplApp_stub( const QCString& app, const QCString& obj )
   : DCOPStub( app, obj )
 {
@@ -38,9 +37,9 @@ void KplApp_stub::newPlot()
     }
 }
 
-bool KplApp_stub::openPlot( const QString& arg0 )
+int KplApp_stub::openPlot( const QString& arg0 )
 {
-    bool result;
+    int result = 0;
     if ( !dcopClient()  ) {
 	setStatus( CallFailed );
 	return result;
@@ -50,7 +49,7 @@ bool KplApp_stub::openPlot( const QString& arg0 )
     QDataStream arg( data, IO_WriteOnly );
     arg << arg0;
     if ( dcopClient()->call( app(), obj(), "openPlot(QString)", data, replyType, replyData ) ) {
-	if ( replyType == "bool" ) {
+	if ( replyType == "int" ) {
 	    QDataStream _reply_stream( replyData, IO_ReadOnly );
 	    _reply_stream >> result;
 	    setStatus( CallSucceeded );
@@ -63,9 +62,9 @@ bool KplApp_stub::openPlot( const QString& arg0 )
     return result;
 }
 
-bool KplApp_stub::openData( const QString& arg0 )
+int KplApp_stub::openData( const QString& arg0 )
 {
-    bool result;
+    int result = 0;
     if ( !dcopClient()  ) {
 	setStatus( CallFailed );
 	return result;
@@ -75,7 +74,7 @@ bool KplApp_stub::openData( const QString& arg0 )
     QDataStream arg( data, IO_WriteOnly );
     arg << arg0;
     if ( dcopClient()->call( app(), obj(), "openData(QString)", data, replyType, replyData ) ) {
-	if ( replyType == "bool" ) {
+	if ( replyType == "int" ) {
 	    QDataStream _reply_stream( replyData, IO_ReadOnly );
 	    _reply_stream >> result;
 	    setStatus( CallSucceeded );
@@ -103,9 +102,9 @@ void KplApp_stub::doPlot()
     }
 }
 
-bool KplApp_stub::savePlot( const QString& arg0 )
+int KplApp_stub::savePlot( const QString& arg0 )
 {
-    bool result;
+    int result = 0;
     if ( !dcopClient()  ) {
 	setStatus( CallFailed );
 	return result;
@@ -115,7 +114,7 @@ bool KplApp_stub::savePlot( const QString& arg0 )
     QDataStream arg( data, IO_WriteOnly );
     arg << arg0;
     if ( dcopClient()->call( app(), obj(), "savePlot(QString)", data, replyType, replyData ) ) {
-	if ( replyType == "bool" ) {
+	if ( replyType == "int" ) {
 	    QDataStream _reply_stream( replyData, IO_ReadOnly );
 	    _reply_stream >> result;
 	    setStatus( CallSucceeded );
@@ -128,9 +127,9 @@ bool KplApp_stub::savePlot( const QString& arg0 )
     return result;
 }
 
-bool KplApp_stub::savePS( const QString& arg0, int arg1 )
+int KplApp_stub::savePS( const QString& arg0, int arg1 )
 {
-    bool result;
+    int result = 0;
     if ( !dcopClient()  ) {
 	setStatus( CallFailed );
 	return result;
@@ -141,7 +140,7 @@ bool KplApp_stub::savePS( const QString& arg0, int arg1 )
     arg << arg0;
     arg << arg1;
     if ( dcopClient()->call( app(), obj(), "savePS(QString,int)", data, replyType, replyData ) ) {
-	if ( replyType == "bool" ) {
+	if ( replyType == "int" ) {
 	    QDataStream _reply_stream( replyData, IO_ReadOnly );
 	    _reply_stream >> result;
 	    setStatus( CallSucceeded );
@@ -154,9 +153,9 @@ bool KplApp_stub::savePS( const QString& arg0, int arg1 )
     return result;
 }
 
-bool KplApp_stub::printPlot( const QString& arg0, const QString& arg1, int arg2, int arg3 )
+int KplApp_stub::printPlot( const QString& arg0, const QString& arg1, int arg2, int arg3 )
 {
-    bool result;
+    int result = 0;
     if ( !dcopClient()  ) {
 	setStatus( CallFailed );
 	return result;
@@ -169,7 +168,7 @@ bool KplApp_stub::printPlot( const QString& arg0, const QString& arg1, int arg2,
     arg << arg2;
     arg << arg3;
     if ( dcopClient()->call( app(), obj(), "printPlot(QString,QString,int,int)", data, replyType, replyData ) ) {
-	if ( replyType == "bool" ) {
+	if ( replyType == "int" ) {
 	    QDataStream _reply_stream( replyData, IO_ReadOnly );
 	    _reply_stream >> result;
 	    setStatus( CallSucceeded );
@@ -368,9 +367,9 @@ void KplApp_stub::setAutofit( const QString& arg0, const QString& arg1, int arg2
     }
 }
 
-bool KplApp_stub::loadAutofitPar( const QString& arg0 )
+int KplApp_stub::loadAutofitPar( const QString& arg0 )
 {
-    bool result;
+    int result = 0;
     if ( !dcopClient()  ) {
 	setStatus( CallFailed );
 	return result;
@@ -380,7 +379,7 @@ bool KplApp_stub::loadAutofitPar( const QString& arg0 )
     QDataStream arg( data, IO_WriteOnly );
     arg << arg0;
     if ( dcopClient()->call( app(), obj(), "loadAutofitPar(QString)", data, replyType, replyData ) ) {
-	if ( replyType == "bool" ) {
+	if ( replyType == "int" ) {
 	    QDataStream _reply_stream( replyData, IO_ReadOnly );
 	    _reply_stream >> result;
 	    setStatus( CallSucceeded );
@@ -393,9 +392,9 @@ bool KplApp_stub::loadAutofitPar( const QString& arg0 )
     return result;
 }
 
-bool KplApp_stub::setAutofitPar( int arg0, double arg1 )
+int KplApp_stub::setAutofitPar( int arg0, double arg1 )
 {
-    bool result;
+    int result = 0;
     if ( !dcopClient()  ) {
 	setStatus( CallFailed );
 	return result;
@@ -406,7 +405,7 @@ bool KplApp_stub::setAutofitPar( int arg0, double arg1 )
     arg << arg0;
     arg << arg1;
     if ( dcopClient()->call( app(), obj(), "setAutofitPar(int,double)", data, replyType, replyData ) ) {
-	if ( replyType == "bool" ) {
+	if ( replyType == "int" ) {
 	    QDataStream _reply_stream( replyData, IO_ReadOnly );
 	    _reply_stream >> result;
 	    setStatus( CallSucceeded );
@@ -419,9 +418,9 @@ bool KplApp_stub::setAutofitPar( int arg0, double arg1 )
     return result;
 }
 
-bool KplApp_stub::enableFitPar( int arg0, int arg1 )
+int KplApp_stub::enableFitPar( int arg0, int arg1 )
 {
-    bool result;
+    int result = 0;
     if ( !dcopClient()  ) {
 	setStatus( CallFailed );
 	return result;
@@ -432,7 +431,7 @@ bool KplApp_stub::enableFitPar( int arg0, int arg1 )
     arg << arg0;
     arg << arg1;
     if ( dcopClient()->call( app(), obj(), "enableFitPar(int,int)", data, replyType, replyData ) ) {
-	if ( replyType == "bool" ) {
+	if ( replyType == "int" ) {
 	    QDataStream _reply_stream( replyData, IO_ReadOnly );
 	    _reply_stream >> result;
 	    setStatus( CallSucceeded );
@@ -445,9 +444,9 @@ bool KplApp_stub::enableFitPar( int arg0, int arg1 )
     return result;
 }
 
-bool KplApp_stub::loadErrModPar( const QString& arg0, int arg1 )
+int KplApp_stub::loadErrModPar( const QString& arg0, int arg1 )
 {
-    bool result;
+    int result = 0;
     if ( !dcopClient()  ) {
 	setStatus( CallFailed );
 	return result;
@@ -458,7 +457,7 @@ bool KplApp_stub::loadErrModPar( const QString& arg0, int arg1 )
     arg << arg0;
     arg << arg1;
     if ( dcopClient()->call( app(), obj(), "loadErrModPar(QString,int)", data, replyType, replyData ) ) {
-	if ( replyType == "bool" ) {
+	if ( replyType == "int" ) {
 	    QDataStream _reply_stream( replyData, IO_ReadOnly );
 	    _reply_stream >> result;
 	    setStatus( CallSucceeded );
@@ -471,9 +470,9 @@ bool KplApp_stub::loadErrModPar( const QString& arg0, int arg1 )
     return result;
 }
 
-bool KplApp_stub::setErrModPar( int arg0, double arg1, int arg2 )
+int KplApp_stub::setErrModPar( int arg0, double arg1, int arg2 )
 {
-    bool result;
+    int result = 0;
     if ( !dcopClient()  ) {
 	setStatus( CallFailed );
 	return result;
@@ -485,7 +484,7 @@ bool KplApp_stub::setErrModPar( int arg0, double arg1, int arg2 )
     arg << arg1;
     arg << arg2;
     if ( dcopClient()->call( app(), obj(), "setErrModPar(int,double,int)", data, replyType, replyData ) ) {
-	if ( replyType == "bool" ) {
+	if ( replyType == "int" ) {
 	    QDataStream _reply_stream( replyData, IO_ReadOnly );
 	    _reply_stream >> result;
 	    setStatus( CallSucceeded );
@@ -498,9 +497,9 @@ bool KplApp_stub::setErrModPar( int arg0, double arg1, int arg2 )
     return result;
 }
 
-bool KplApp_stub::deleteItem( int arg0 )
+int KplApp_stub::deleteItem( int arg0 )
 {
-    bool result;
+    int result = 0;
     if ( !dcopClient()  ) {
 	setStatus( CallFailed );
 	return result;
@@ -510,7 +509,7 @@ bool KplApp_stub::deleteItem( int arg0 )
     QDataStream arg( data, IO_WriteOnly );
     arg << arg0;
     if ( dcopClient()->call( app(), obj(), "deleteItem(int)", data, replyType, replyData ) ) {
-	if ( replyType == "bool" ) {
+	if ( replyType == "int" ) {
 	    QDataStream _reply_stream( replyData, IO_ReadOnly );
 	    _reply_stream >> result;
 	    setStatus( CallSucceeded );
@@ -523,9 +522,9 @@ bool KplApp_stub::deleteItem( int arg0 )
     return result;
 }
 
-bool KplApp_stub::moveItem( int arg0, int arg1 )
+int KplApp_stub::moveItem( int arg0, int arg1 )
 {
-    bool result;
+    int result = 0;
     if ( !dcopClient()  ) {
 	setStatus( CallFailed );
 	return result;
@@ -536,7 +535,33 @@ bool KplApp_stub::moveItem( int arg0, int arg1 )
     arg << arg0;
     arg << arg1;
     if ( dcopClient()->call( app(), obj(), "moveItem(int,int)", data, replyType, replyData ) ) {
-	if ( replyType == "bool" ) {
+	if ( replyType == "int" ) {
+	    QDataStream _reply_stream( replyData, IO_ReadOnly );
+	    _reply_stream >> result;
+	    setStatus( CallSucceeded );
+	} else {
+	    callFailed();
+	}
+    } else { 
+	callFailed();
+    }
+    return result;
+}
+
+int KplApp_stub::activateItem( int arg0, int arg1 )
+{
+    int result = 0;
+    if ( !dcopClient()  ) {
+	setStatus( CallFailed );
+	return result;
+    }
+    QByteArray data, replyData;
+    QCString replyType;
+    QDataStream arg( data, IO_WriteOnly );
+    arg << arg0;
+    arg << arg1;
+    if ( dcopClient()->call( app(), obj(), "activateItem(int,int)", data, replyType, replyData ) ) {
+	if ( replyType == "int" ) {
 	    QDataStream _reply_stream( replyData, IO_ReadOnly );
 	    _reply_stream >> result;
 	    setStatus( CallSucceeded );
@@ -606,9 +631,9 @@ void KplApp_stub::newAutoFrameItem()
     }
 }
 
-bool KplApp_stub::newArrayItem( const QString& arg0, int arg1, int arg2, int arg3, int arg4, int arg5, int arg6, int arg7, const QString& arg8, double arg9, double arg10, int arg11 )
+int KplApp_stub::newArrayItem( const QString& arg0, int arg1, int arg2, int arg3, int arg4, int arg5, int arg6, int arg7, const QString& arg8, double arg9, double arg10, int arg11 )
 {
-    bool result;
+    int result = 0;
     if ( !dcopClient()  ) {
 	setStatus( CallFailed );
 	return result;
@@ -629,7 +654,7 @@ bool KplApp_stub::newArrayItem( const QString& arg0, int arg1, int arg2, int arg
     arg << arg10;
     arg << arg11;
     if ( dcopClient()->call( app(), obj(), "newArrayItem(QString,int,int,int,int,int,int,int,QString,double,double,int)", data, replyType, replyData ) ) {
-	if ( replyType == "bool" ) {
+	if ( replyType == "int" ) {
 	    QDataStream _reply_stream( replyData, IO_ReadOnly );
 	    _reply_stream >> result;
 	    setStatus( CallSucceeded );
@@ -642,9 +667,9 @@ bool KplApp_stub::newArrayItem( const QString& arg0, int arg1, int arg2, int arg
     return result;
 }
 
-bool KplApp_stub::newFunItem( const QString& arg0, const QString& arg1, double arg2, double arg3, double arg4, int arg5, const QString& arg6, double arg7, double arg8, int arg9 )
+int KplApp_stub::newFunItem( const QString& arg0, const QString& arg1, double arg2, double arg3, double arg4, int arg5, const QString& arg6, double arg7, double arg8, int arg9 )
 {
-    bool result;
+    int result = 0;
     if ( !dcopClient()  ) {
 	setStatus( CallFailed );
 	return result;
@@ -663,7 +688,7 @@ bool KplApp_stub::newFunItem( const QString& arg0, const QString& arg1, double a
     arg << arg8;
     arg << arg9;
     if ( dcopClient()->call( app(), obj(), "newFunItem(QString,QString,double,double,double,int,QString,double,double,int)", data, replyType, replyData ) ) {
-	if ( replyType == "bool" ) {
+	if ( replyType == "int" ) {
 	    QDataStream _reply_stream( replyData, IO_ReadOnly );
 	    _reply_stream >> result;
 	    setStatus( CallSucceeded );
@@ -676,9 +701,9 @@ bool KplApp_stub::newFunItem( const QString& arg0, const QString& arg1, double a
     return result;
 }
 
-bool KplApp_stub::newParFunItem( const QString& arg0, const QString& arg1, const QString& arg2, const QString& arg3, double arg4, double arg5, double arg6, int arg7, const QString& arg8, double arg9, double arg10, int arg11 )
+int KplApp_stub::newParFunItem( const QString& arg0, const QString& arg1, const QString& arg2, const QString& arg3, double arg4, double arg5, double arg6, int arg7, const QString& arg8, double arg9, double arg10, int arg11 )
 {
-    bool result;
+    int result = 0;
     if ( !dcopClient()  ) {
 	setStatus( CallFailed );
 	return result;
@@ -699,7 +724,191 @@ bool KplApp_stub::newParFunItem( const QString& arg0, const QString& arg1, const
     arg << arg10;
     arg << arg11;
     if ( dcopClient()->call( app(), obj(), "newParFunItem(QString,QString,QString,QString,double,double,double,int,QString,double,double,int)", data, replyType, replyData ) ) {
-	if ( replyType == "bool" ) {
+	if ( replyType == "int" ) {
+	    QDataStream _reply_stream( replyData, IO_ReadOnly );
+	    _reply_stream >> result;
+	    setStatus( CallSucceeded );
+	} else {
+	    callFailed();
+	}
+    } else { 
+	callFailed();
+    }
+    return result;
+}
+
+void KplApp_stub::newSplineItem( int arg0, double arg1, double arg2, double arg3, double arg4, int arg5, const QString& arg6, double arg7, double arg8, double arg9, int arg10 )
+{
+    if ( !dcopClient()  ) {
+	setStatus( CallFailed );
+	return;
+    }
+    QByteArray data, replyData;
+    QCString replyType;
+    QDataStream arg( data, IO_WriteOnly );
+    arg << arg0;
+    arg << arg1;
+    arg << arg2;
+    arg << arg3;
+    arg << arg4;
+    arg << arg5;
+    arg << arg6;
+    arg << arg7;
+    arg << arg8;
+    arg << arg9;
+    arg << arg10;
+    if ( dcopClient()->call( app(), obj(), "newSplineItem(int,double,double,double,double,int,QString,double,double,double,int)", data, replyType, replyData ) ) {
+	setStatus( CallSucceeded );
+    } else { 
+	callFailed();
+    }
+}
+
+int KplApp_stub::newArray3DItem( double arg0, double arg1, double arg2, double arg3, double arg4, double arg5, double arg6, double arg7, double arg8, double arg9, int arg10, int arg11, int arg12, int arg13, int arg14, int arg15, const QString& arg16, const QString& arg17, const QString& arg18, double arg19, double arg20, double arg21, int arg22, int arg23, int arg24, int arg25, int arg26, int arg27, double arg28, double arg29, double arg30, double arg31, double arg32, double arg33, double arg34, int arg35, int arg36, int arg37, const QString& arg38, const QString& arg39, const QString& arg40, const QString& arg41, double arg42, const QString& arg43, int arg44, int arg45, int arg46, int arg47, int arg48, int arg49, int arg50, double arg51, double arg52, double arg53, double arg54, double arg55, double arg56, int arg57 )
+{
+    int result = 0;
+    if ( !dcopClient()  ) {
+	setStatus( CallFailed );
+	return result;
+    }
+    QByteArray data, replyData;
+    QCString replyType;
+    QDataStream arg( data, IO_WriteOnly );
+    arg << arg0;
+    arg << arg1;
+    arg << arg2;
+    arg << arg3;
+    arg << arg4;
+    arg << arg5;
+    arg << arg6;
+    arg << arg7;
+    arg << arg8;
+    arg << arg9;
+    arg << arg10;
+    arg << arg11;
+    arg << arg12;
+    arg << arg13;
+    arg << arg14;
+    arg << arg15;
+    arg << arg16;
+    arg << arg17;
+    arg << arg18;
+    arg << arg19;
+    arg << arg20;
+    arg << arg21;
+    arg << arg22;
+    arg << arg23;
+    arg << arg24;
+    arg << arg25;
+    arg << arg26;
+    arg << arg27;
+    arg << arg28;
+    arg << arg29;
+    arg << arg30;
+    arg << arg31;
+    arg << arg32;
+    arg << arg33;
+    arg << arg34;
+    arg << arg35;
+    arg << arg36;
+    arg << arg37;
+    arg << arg38;
+    arg << arg39;
+    arg << arg40;
+    arg << arg41;
+    arg << arg42;
+    arg << arg43;
+    arg << arg44;
+    arg << arg45;
+    arg << arg46;
+    arg << arg47;
+    arg << arg48;
+    arg << arg49;
+    arg << arg50;
+    arg << arg51;
+    arg << arg52;
+    arg << arg53;
+    arg << arg54;
+    arg << arg55;
+    arg << arg56;
+    arg << arg57;
+    if ( dcopClient()->call( app(), obj(), "newArray3DItem(double,double,double,double,double,double,double,double,double,double,int,int,int,int,int,int,QString,QString,QString,double,double,double,int,int,int,int,int,int,double,double,double,double,double,double,double,int,int,int,QString,QString,QString,QString,double,QString,int,int,int,int,int,int,int,double,double,double,double,double,double,int)", data, replyType, replyData ) ) {
+	if ( replyType == "int" ) {
+	    QDataStream _reply_stream( replyData, IO_ReadOnly );
+	    _reply_stream >> result;
+	    setStatus( CallSucceeded );
+	} else {
+	    callFailed();
+	}
+    } else { 
+	callFailed();
+    }
+    return result;
+}
+
+int KplApp_stub::newFun3DItem( double arg0, double arg1, double arg2, double arg3, double arg4, double arg5, double arg6, double arg7, double arg8, double arg9, int arg10, int arg11, int arg12, int arg13, int arg14, int arg15, const QString& arg16, const QString& arg17, const QString& arg18, double arg19, double arg20, double arg21, int arg22, int arg23, int arg24, int arg25, int arg26, int arg27, double arg28, double arg29, double arg30, double arg31, double arg32, double arg33, double arg34, int arg35, int arg36, int arg37, const QString& arg38, const QString& arg39, const QString& arg40, const QString& arg41, double arg42, const QString& arg43, const QString& arg44, double arg45, double arg46, double arg47, double arg48, double arg49, int arg50 )
+{
+    int result = 0;
+    if ( !dcopClient()  ) {
+	setStatus( CallFailed );
+	return result;
+    }
+    QByteArray data, replyData;
+    QCString replyType;
+    QDataStream arg( data, IO_WriteOnly );
+    arg << arg0;
+    arg << arg1;
+    arg << arg2;
+    arg << arg3;
+    arg << arg4;
+    arg << arg5;
+    arg << arg6;
+    arg << arg7;
+    arg << arg8;
+    arg << arg9;
+    arg << arg10;
+    arg << arg11;
+    arg << arg12;
+    arg << arg13;
+    arg << arg14;
+    arg << arg15;
+    arg << arg16;
+    arg << arg17;
+    arg << arg18;
+    arg << arg19;
+    arg << arg20;
+    arg << arg21;
+    arg << arg22;
+    arg << arg23;
+    arg << arg24;
+    arg << arg25;
+    arg << arg26;
+    arg << arg27;
+    arg << arg28;
+    arg << arg29;
+    arg << arg30;
+    arg << arg31;
+    arg << arg32;
+    arg << arg33;
+    arg << arg34;
+    arg << arg35;
+    arg << arg36;
+    arg << arg37;
+    arg << arg38;
+    arg << arg39;
+    arg << arg40;
+    arg << arg41;
+    arg << arg42;
+    arg << arg43;
+    arg << arg44;
+    arg << arg45;
+    arg << arg46;
+    arg << arg47;
+    arg << arg48;
+    arg << arg49;
+    arg << arg50;
+    if ( dcopClient()->call( app(), obj(), "newFun3DItem(double,double,double,double,double,double,double,double,double,double,int,int,int,int,int,int,QString,QString,QString,double,double,double,int,int,int,int,int,int,double,double,double,double,double,double,double,int,int,int,QString,QString,QString,QString,double,QString,QString,double,double,double,double,double,int)", data, replyType, replyData ) ) {
+	if ( replyType == "int" ) {
 	    QDataStream _reply_stream( replyData, IO_ReadOnly );
 	    _reply_stream >> result;
 	    setStatus( CallSucceeded );
@@ -809,6 +1018,32 @@ void KplApp_stub::newArrowItem( const QString& arg0, double arg1, double arg2, d
     }
 }
 
+void KplApp_stub::newArcItem( const QString& arg0, double arg1, double arg2, double arg3, double arg4, double arg5, double arg6, double arg7, double arg8, int arg9 )
+{
+    if ( !dcopClient()  ) {
+	setStatus( CallFailed );
+	return;
+    }
+    QByteArray data, replyData;
+    QCString replyType;
+    QDataStream arg( data, IO_WriteOnly );
+    arg << arg0;
+    arg << arg1;
+    arg << arg2;
+    arg << arg3;
+    arg << arg4;
+    arg << arg5;
+    arg << arg6;
+    arg << arg7;
+    arg << arg8;
+    arg << arg9;
+    if ( dcopClient()->call( app(), obj(), "newArcItem(QString,double,double,double,double,double,double,double,double,int)", data, replyType, replyData ) ) {
+	setStatus( CallSucceeded );
+    } else { 
+	callFailed();
+    }
+}
+
 void KplApp_stub::newRectangleItem( const QString& arg0, double arg1, double arg2, double arg3, double arg4, double arg5, int arg6, int arg7 )
 {
     if ( !dcopClient()  ) {
@@ -857,9 +1092,9 @@ void KplApp_stub::newEllipseItem( const QString& arg0, double arg1, double arg2,
     }
 }
 
-bool KplApp_stub::autoScaleFrame( int arg0, int arg1 )
+int KplApp_stub::autoScaleFrame( int arg0, int arg1 )
 {
-    bool result;
+    int result = 0;
     if ( !dcopClient()  ) {
 	setStatus( CallFailed );
 	return result;
@@ -870,7 +1105,7 @@ bool KplApp_stub::autoScaleFrame( int arg0, int arg1 )
     arg << arg0;
     arg << arg1;
     if ( dcopClient()->call( app(), obj(), "autoScaleFrame(int,int)", data, replyType, replyData ) ) {
-	if ( replyType == "bool" ) {
+	if ( replyType == "int" ) {
 	    QDataStream _reply_stream( replyData, IO_ReadOnly );
 	    _reply_stream >> result;
 	    setStatus( CallSucceeded );
@@ -883,9 +1118,9 @@ bool KplApp_stub::autoScaleFrame( int arg0, int arg1 )
     return result;
 }
 
-bool KplApp_stub::loadPar( const QString& arg0, int arg1, int arg2 )
+int KplApp_stub::loadPar( const QString& arg0, int arg1, int arg2 )
 {
-    bool result;
+    int result = 0;
     if ( !dcopClient()  ) {
 	setStatus( CallFailed );
 	return result;
@@ -897,7 +1132,7 @@ bool KplApp_stub::loadPar( const QString& arg0, int arg1, int arg2 )
     arg << arg1;
     arg << arg2;
     if ( dcopClient()->call( app(), obj(), "loadPar(QString,int,int)", data, replyType, replyData ) ) {
-	if ( replyType == "bool" ) {
+	if ( replyType == "int" ) {
 	    QDataStream _reply_stream( replyData, IO_ReadOnly );
 	    _reply_stream >> result;
 	    setStatus( CallSucceeded );
@@ -910,9 +1145,9 @@ bool KplApp_stub::loadPar( const QString& arg0, int arg1, int arg2 )
     return result;
 }
 
-bool KplApp_stub::setPar( int arg0, int arg1, double arg2, int arg3 )
+int KplApp_stub::setPar( int arg0, int arg1, double arg2, int arg3 )
 {
-    bool result;
+    int result = 0;
     if ( !dcopClient()  ) {
 	setStatus( CallFailed );
 	return result;
@@ -925,7 +1160,7 @@ bool KplApp_stub::setPar( int arg0, int arg1, double arg2, int arg3 )
     arg << arg2;
     arg << arg3;
     if ( dcopClient()->call( app(), obj(), "setPar(int,int,double,int)", data, replyType, replyData ) ) {
-	if ( replyType == "bool" ) {
+	if ( replyType == "int" ) {
 	    QDataStream _reply_stream( replyData, IO_ReadOnly );
 	    _reply_stream >> result;
 	    setStatus( CallSucceeded );
@@ -938,9 +1173,9 @@ bool KplApp_stub::setPar( int arg0, int arg1, double arg2, int arg3 )
     return result;
 }
 
-bool KplApp_stub::setArrayRange( int arg0, int arg1, int arg2 )
+int KplApp_stub::setArrayRange( int arg0, int arg1, int arg2 )
 {
-    bool result;
+    int result = 0;
     if ( !dcopClient()  ) {
 	setStatus( CallFailed );
 	return result;
@@ -952,7 +1187,7 @@ bool KplApp_stub::setArrayRange( int arg0, int arg1, int arg2 )
     arg << arg1;
     arg << arg2;
     if ( dcopClient()->call( app(), obj(), "setArrayRange(int,int,int)", data, replyType, replyData ) ) {
-	if ( replyType == "bool" ) {
+	if ( replyType == "int" ) {
 	    QDataStream _reply_stream( replyData, IO_ReadOnly );
 	    _reply_stream >> result;
 	    setStatus( CallSucceeded );
@@ -965,9 +1200,9 @@ bool KplApp_stub::setArrayRange( int arg0, int arg1, int arg2 )
     return result;
 }
 
-bool KplApp_stub::addFitItems( int arg0, int arg1, int arg2, int arg3, const QString& arg4, const QString& arg5 )
+int KplApp_stub::addFitItems( int arg0, int arg1, int arg2, int arg3, const QString& arg4, const QString& arg5 )
 {
-    bool result;
+    int result = 0;
     if ( !dcopClient()  ) {
 	setStatus( CallFailed );
 	return result;
@@ -982,7 +1217,7 @@ bool KplApp_stub::addFitItems( int arg0, int arg1, int arg2, int arg3, const QSt
     arg << arg4;
     arg << arg5;
     if ( dcopClient()->call( app(), obj(), "addFitItems(int,int,int,int,QString,QString)", data, replyType, replyData ) ) {
-	if ( replyType == "bool" ) {
+	if ( replyType == "int" ) {
 	    QDataStream _reply_stream( replyData, IO_ReadOnly );
 	    _reply_stream >> result;
 	    setStatus( CallSucceeded );
@@ -995,9 +1230,9 @@ bool KplApp_stub::addFitItems( int arg0, int arg1, int arg2, int arg3, const QSt
     return result;
 }
 
-bool KplApp_stub::fit( int arg0, int arg1, int arg2 )
+int KplApp_stub::fit( int arg0, int arg1, int arg2 )
 {
-    bool result;
+    int result = 0;
     if ( !dcopClient()  ) {
 	setStatus( CallFailed );
 	return result;
@@ -1009,7 +1244,40 @@ bool KplApp_stub::fit( int arg0, int arg1, int arg2 )
     arg << arg1;
     arg << arg2;
     if ( dcopClient()->call( app(), obj(), "fit(int,int,int)", data, replyType, replyData ) ) {
-	if ( replyType == "bool" ) {
+	if ( replyType == "int" ) {
+	    QDataStream _reply_stream( replyData, IO_ReadOnly );
+	    _reply_stream >> result;
+	    setStatus( CallSucceeded );
+	} else {
+	    callFailed();
+	}
+    } else { 
+	callFailed();
+    }
+    return result;
+}
+
+int KplApp_stub::splineFit( int arg0, int arg1, int arg2, double arg3, double arg4, double arg5, int arg6, const QString& arg7, const QString& arg8 )
+{
+    int result = 0;
+    if ( !dcopClient()  ) {
+	setStatus( CallFailed );
+	return result;
+    }
+    QByteArray data, replyData;
+    QCString replyType;
+    QDataStream arg( data, IO_WriteOnly );
+    arg << arg0;
+    arg << arg1;
+    arg << arg2;
+    arg << arg3;
+    arg << arg4;
+    arg << arg5;
+    arg << arg6;
+    arg << arg7;
+    arg << arg8;
+    if ( dcopClient()->call( app(), obj(), "splineFit(int,int,int,double,double,double,int,QString,QString)", data, replyType, replyData ) ) {
+	if ( replyType == "int" ) {
 	    QDataStream _reply_stream( replyData, IO_ReadOnly );
 	    _reply_stream >> result;
 	    setStatus( CallSucceeded );
@@ -1045,9 +1313,9 @@ double KplApp_stub::chiSquare()
     return result;
 }
 
-bool KplApp_stub::saveFitPar( const QString& arg0 )
+int KplApp_stub::saveFitPar( const QString& arg0 )
 {
-    bool result;
+    int result = 0;
     if ( !dcopClient()  ) {
 	setStatus( CallFailed );
 	return result;
@@ -1057,7 +1325,7 @@ bool KplApp_stub::saveFitPar( const QString& arg0 )
     QDataStream arg( data, IO_WriteOnly );
     arg << arg0;
     if ( dcopClient()->call( app(), obj(), "saveFitPar(QString)", data, replyType, replyData ) ) {
-	if ( replyType == "bool" ) {
+	if ( replyType == "int" ) {
 	    QDataStream _reply_stream( replyData, IO_ReadOnly );
 	    _reply_stream >> result;
 	    setStatus( CallSucceeded );
@@ -1070,9 +1338,9 @@ bool KplApp_stub::saveFitPar( const QString& arg0 )
     return result;
 }
 
-bool KplApp_stub::exportValues( const QString& arg0, int arg1 )
+int KplApp_stub::exportValues( const QString& arg0, int arg1 )
 {
-    bool result;
+    int result = 0;
     if ( !dcopClient()  ) {
 	setStatus( CallFailed );
 	return result;
@@ -1083,7 +1351,7 @@ bool KplApp_stub::exportValues( const QString& arg0, int arg1 )
     arg << arg0;
     arg << arg1;
     if ( dcopClient()->call( app(), obj(), "exportValues(QString,int)", data, replyType, replyData ) ) {
-	if ( replyType == "bool" ) {
+	if ( replyType == "int" ) {
 	    QDataStream _reply_stream( replyData, IO_ReadOnly );
 	    _reply_stream >> result;
 	    setStatus( CallSucceeded );
