@@ -6,7 +6,7 @@
     data sets and functions.
 
     begin                : Wed Sep 08 1999
-    copyright            : (C) 2001 by Werner Stille
+    copyright            : (C) 2002 by Werner Stille
     email                : stille@uni-freiburg.de
  ***************************************************************************/
 
@@ -19,6 +19,7 @@
  *                                                                         *
  ***************************************************************************/
 
+#include <qcolor.h>
 #include "kplgraph.h"
 
 KplGraph::KplGraph() : logx(false), logy(false), kSymb(0), relSize(1.0),
@@ -34,3 +35,25 @@ KplGraph::DevTypes KplGraph::type() const
 {
   return devType;
 }
+
+int KplGraph::prenum(double a, int ndig, QString& label)
+{
+  label.sprintf("%.*f", QMAX(0, ndig), a);
+  if (ndig == 0)
+    label += ".";
+  return label.length();
+}
+
+unsigned KplGraph::rgbQt(unsigned u)
+{
+  QColor col(int(u & 0xffL), int((u & 0xff00L) >> 8),
+             int((u & 0xff0000L) >> 16));
+  return col.rgb();
+}
+
+unsigned KplGraph::rgbQt1(unsigned rgb)
+{
+  QColor col(rgb);
+  return unsigned(col.red()) | (unsigned(col.green()) << 8) |
+    (unsigned(col.blue()) << 16);
+}
