@@ -2,7 +2,7 @@
                           kplspinbox.cpp  -  description
                              -------------------
     begin                : Fri Sep 8 2000
-    copyright            : (C) 2001 by Werner Stille
+    copyright            : (C) 2002 by Werner Stille
     email                : stille@uni-freiburg.de
  ***************************************************************************/
 
@@ -17,9 +17,9 @@
 
 #include "kplspinbox.h"
 
-KplSpinBox::KplSpinBox(int minValue, int maxValue, int step,
-                       QWidget* parent, const char* name) :
- QSpinBox(minValue, maxValue, step, parent, name)
+KplSpinBox::KplSpinBox(int _minValue, int _maxValue, int step,
+                       QWidget* _parent, const char* _name) :
+ QSpinBox(_minValue, _maxValue, step, _parent, _name)
 {
 }
 
