@@ -25,16 +25,11 @@
 #include "arrayitem.h"
 #include "funitem.h"
 
-/* resolution of arithmetic */
-#define MACHEP DBL_EPSILON
-/* smallest nonzero number */
-#define DWARF 1.0e-38
-
 /*********************** lmdif.c ****************************/
 #define BUG 0
 
-LMFit::LMFit(QObject* parent) : QObject(parent), bFit(0), userBreak(false),
- sig(0), p(0)
+LMFit::LMFit(QObject* _parent) : QObject(_parent), bFit(0), userBreak(false),
+ p(0)
 {
 }
 
@@ -52,15 +47,16 @@ bool LMFit::linFit(int nd, int np, double px[], double fvec[], double a[],
   int iflag = 0;
   fcn(nd, np, pt, wa4, &iflag);
   if (!iflag) {
+    double dx = sqrt(DBL_MAX);
     ij = jj = 0;
     for (j = 0; j < KPL_NPMAX; j++)
       if (bFit[j]) {
-        pt[jj] = 1.0;
+        pt[jj] = dx;
         fcn(nd, np, pt, &a[ij], &iflag);
         if (iflag)
           break;
         for (i = 0; i < nd; i++)
-          a[ij++] = wa4[i] - a[ij];
+          a[ij++] = (wa4[i] - a[ij]) / dx;
         pt[jj++] = 0.0;
       }
   }
@@ -97,30 +93,6 @@ bool LMFit::linFit(int nd, int np, double px[], double fvec[], double a[],
   return (iflag);
 }
 
-void LMFit::fcn(int, int, const double a[], double fvec[], int* iflag)
-{
-  kapp->processEvents();
-  if (userBreak) {
-    *iflag = -1;
-    return;
-  }
-  int j = 0;
-  int i;
-  for (i = 0; i < KPL_NPMAX; i++)
-    if (bFit[i])
-      p[i] = a[j++];
-  int k = 0;
-  for (i = 0; i < (int) ad->count(); i++) {
-    ArrayItem *ar = ad->at(i);
-    double (*f)(double, const double *) = fd->at(i)->fkty;
-    for (j = 0; j < ar->n; j++) {
-      fvec[k] = (ar->x[ar->iy][ar->istart + j] -
-        (*f)(ar->x[ar->ix][ar->istart + j], p)) / sig[k];
-      k++;
-    }
-  }
-}
-
 void LMFit::lmdif(int m, int n, double x[], double fvec[], double ftol,
                   double xtol, double gtol, int maxfev, double epsfcn,
                   double diag[], int mode, double factor, int nprint,
@@ -634,13 +606,13 @@ if( *info != 0)
 */
 if( *nfev >= maxfev)
 	*info = 5;
-if( (fabs(actred) <= MACHEP)
-  && (prered <= MACHEP)
+if( (fabs(actred) <= DBL_EPSILON)
+  && (prered <= DBL_EPSILON)
   && (p5*ratio <= one) )
 	*info = 6;
-if(delta <= MACHEP*xnorm)
+if(delta <= DBL_EPSILON*xnorm)
 	*info = 7;
-if(gnorm <= MACHEP)
+if(gnorm <= DBL_EPSILON)
 	*info = 8;
 if( *info != 0)
 	goto L300;
@@ -668,133 +640,693 @@ if(nprint > 0)
 */
 }
 
-/************************fdjac2.c*************************/
+/************************enorm.c*************************/
 
-#define BUG 0
-
-void LMFit::fdjac2(int m, int n, double x[], const double fvec[], double fjac[],
-                   int, int *iflag, double epsfcn, double wa[])
+double LMFit::enorm(int n, const double x[])
 {
 /*
 *     **********
 *
-*     subroutine fdjac2
+*     function enorm
 *
-*     this subroutine computes a forward-difference approximation
-*     to the m by n jacobian matrix associated with a specified
-*     problem of m functions in n variables.
+*     given an n-vector x, this function calculates the
+*     euclidean norm of x.
 *
-*     the subroutine statement is
+*     the euclidean norm is computed by accumulating the sum of
+*     squares in three different sums. the sums of squares for the
+*     small and large components are scaled so that no overflows
+*     occur. non-destructive underflows are permitted. underflows
+*     and overflows do not occur in the computation of the unscaled
+*     sum of squares for the intermediate components.
+*     the definitions of small, intermediate and large components
+*     depend on two constants, rdwarf and rgiant. the main
+*     restrictions on these constants are that rdwarf**2 not
+*     underflow and rgiant**2 not overflow. the constants
+*     given here are suitable for every known computer.
 *
-*	subroutine fdjac2(fcn,m,n,x,fvec,fjac,ldfjac,iflag,epsfcn,wa)
+*     the function statement is
+*
+*	double precision function enorm(n,x)
 *
 *     where
 *
-*	fcn is the name of the user-supplied subroutine which
-*	  calculates the functions. fcn must be declared
-*	  in an external statement in the user calling
-*	  program, and should be written as follows.
-*
-*	  subroutine fcn(m,n,x,fvec,iflag)
-*	  integer m,n,iflag
-*	  double precision x(n),fvec(m)
-*	  ----------
-*	  calculate the functions at x and
-*	  return this vector in fvec.
-*	  ----------
-*	  return
-*	  end
-*
-*	  the value of iflag should not be changed by fcn unless
-*	  the user wants to terminate execution of fdjac2.
-*	  in this case set iflag to a negative integer.
-*
-*	m is a positive integer input variable set to the number
-*	  of functions.
-*
-*	n is a positive integer input variable set to the number
-*	  of variables. n must not exceed m.
+*	n is a positive integer input variable.
 *
 *	x is an input array of length n.
 *
-*	fvec is an input array of length m which must contain the
-*	  functions evaluated at x.
-*
-*	fjac is an output m by n array which contains the
-*	  approximation to the jacobian matrix evaluated at x.
-*
-*	ldfjac is a positive integer input variable not less than m
-*	  which specifies the leading dimension of the array fjac.
-*
-*	iflag is an integer variable which can be used to terminate
-*	  the execution of fdjac2. see description of fcn.
-*
-*	epsfcn is an input variable used in determining a suitable
-*	  step length for the forward-difference approximation. this
-*	  approximation assumes that the relative errors in the
-*	  functions are of the order of epsfcn. if epsfcn is less
-*	  than the machine precision, it is assumed that the relative
-*	  errors in the functions are of the order of the machine
-*	  precision.
-*
-*	wa is a work array of length m.
-*
 *     subprograms called
 *
-*	user-supplied ...... fcn
-*
-*	minpack-supplied ... dpmpar
-*
-*	fortran-supplied ... dabs,dmax1,dsqrt
+*	fortran-supplied ... dabs,dsqrt
 *
 *     argonne national laboratory. minpack project. march 1980.
 *     burton s. garbow, kenneth e. hillstrom, jorge j. more
 *
-      **********
+*     **********
 */
-int i,j,ij;
-double eps,h,temp;
+int i;
+double agiant,floatn,s1,s2,s3,xabs,x1max,x3max;
+double ans, temp;
+static double rdwarf = sqrt(DBL_MIN);
+static double rgiant = sqrt(DBL_MAX);
 static double zero = 0.0;
+static double one = 1.0;
 
-temp = QMAX(epsfcn,MACHEP);
-eps = sqrt(temp);
-#if BUG
-printf( "fdjac2\n" );
-#endif
-ij = 0;
-for( j=0; j<n; j++ )
+s1 = zero;
+s2 = zero;
+s3 = zero;
+x1max = zero;
+x3max = zero;
+floatn = n;
+agiant = rgiant/floatn;
+
+for( i=0; i<n; i++ )
+{
+xabs = fabs(x[i]);
+if( (xabs > rdwarf) && (xabs < agiant) )
 	{
-	temp = x[j];
-	h = eps * fabs(temp);
-	if(h == zero)
-		h = eps;
-	x[j] = temp + h;
-	fcn(m,n,x,wa,iflag);
-	if( *iflag < 0)
-		return;
-	x[j] = temp;
-	for( i=0; i<m; i++ )
+/*
+*	    sum for intermediate components.
+*/
+	s2 += xabs*xabs;
+	continue;
+	}
+
+if(xabs > rdwarf)
+	{
+/*
+*	       sum for large components.
+*/
+	if(xabs > x1max)
 		{
-		fjac[ij] = (wa[i] - fvec[i])/h;
-		ij += 1;	/* fjac[i+m*j] */
+		temp = x1max/xabs;
+		s1 = one + s1*temp*temp;
+		x1max = xabs;
+		}
+	else
+		{
+		temp = xabs/x1max;
+		s1 += temp*temp;
+		}
+	continue;
+	}
+/*
+*	       sum for small components.
+*/
+if(xabs > x3max)
+	{
+	temp = x3max/xabs;
+	s3 = one + s3*temp*temp;
+	x3max = xabs;
+	}
+else	
+	{
+	if(xabs != zero)
+		{
+		temp = xabs/x3max;
+		s3 += temp*temp;
 		}
 	}
-#if BUG
-pmat( m, n, fjac );
-#endif
+}
 /*
-*     last card of subroutine fdjac2.
+*     calculation of norm.
 */
+if(s1 != zero)
+	{
+	temp = s1 + (s2/x1max)/x1max;
+	ans = x1max*sqrt(temp);
+	return(ans);
+	}
+if(s2 != zero)
+	{
+	if(s2 >= x3max)
+		temp = s2*(one+(x3max/s2)*(x3max*s3));
+	else
+		temp = x3max*((s2/x3max)+(x3max*s3));
+	ans = sqrt(temp);
+	}
+else
+	{
+	ans = x3max*sqrt(s3);
+	}
+return(ans);
+/*
+*     last card of function enorm.
+*/
+}
+
+/*							minv.c
+ *
+ *	Matrix inversion
+ *
+ *
+ *
+ * SYNOPSIS:
+ *
+ * int n, errcod;
+ * double A[n*n], X[n*n];
+ * double B[n];
+ * int IPS[n];
+ * int minv();
+ *
+ * errcod = minv( A, X, n, B, IPS );
+ *
+ *
+ *
+ * DESCRIPTION:
+ *
+ * Finds the inverse of the n by n matrix A.  The result goes
+ * to X.   B and IPS are scratch pad arrays of length n.
+ * The contents of matrix A are destroyed.
+ *
+ * The routine returns nonzero on error; error messages are printed
+ * by subroutine simq().
+ *
+ */
+
+int LMFit::minv(double A[], double X[], int n, double B[], int IPS[])
+{
+double *pX;
+int i, k;
+
+for( i=1; i<n; i++ )
+	B[i] = 0.0;
+B[0] = 1.0;
+/* Reduce the matrix and solve for first right hand side vector */
+pX = X;
+k = simq( A, B, pX, n, 1, IPS );
+if( k )
+	return(-1);
+/* Solve for the remaining right hand side vectors */
+for( i=1; i<n; i++ )
+	{
+	B[i-1] = 0.0;
+	B[i] = 1.0;
+	pX += n;
+	k = simq( A, B, pX, n, -1, IPS );
+	if( k )
+		return(-1);
+	}
+/* Transpose the array of solution vectors */
+mtransp( n, X, X );
+return(0);
+}
+
+/*							igam.c
+ *
+ *	Incomplete gamma integral
+ *
+ *
+ *
+ * SYNOPSIS:
+ *
+ * double a, x, y, igam();
+ *
+ * y = igam( a, x );
+ *
+ * DESCRIPTION:
+ *
+ * The function is defined by
+ *
+ *                           x
+ *                            -
+ *                   1       | |  -t  a-1
+ *  igam(a,x)  =   -----     |   e   t   dt.
+ *                  -      | |
+ *                 | (a)    -
+ *                           0
+ *
+ *
+ * In this implementation both arguments must be positive.
+ * The integral is evaluated by either a power series or
+ * continued fraction expansion, depending on the relative
+ * values of a and x.
+ *
+ * ACCURACY:
+ *
+ *                      Relative error:
+ * arithmetic   domain     # trials      peak         rms
+ *    IEEE      0,30       200000       3.6e-14     2.9e-15
+ *    IEEE      0,100      300000       9.9e-14     1.5e-14
+ */
+/*							igamc()
+ *
+ *	Complemented incomplete gamma integral
+ *
+ *
+ *
+ * SYNOPSIS:
+ *
+ * double a, x, y, igamc();
+ *
+ * y = igamc( a, x );
+ *
+ * DESCRIPTION:
+ *
+ * The function is defined by
+ *
+ *
+ *  igamc(a,x)   =   1 - igam(a,x)
+ *
+ *                            inf.
+ *                              -
+ *                     1       | |  -t  a-1
+ *               =   -----     |   e   t   dt.
+ *                    -      | |
+ *                   | (a)    -
+ *                             x
+ *
+ *
+ * In this implementation both arguments must be positive.
+ * The integral is evaluated by either a power series or
+ * continued fraction expansion, depending on the relative
+ * values of a and x.
+ *
+ * ACCURACY:
+ *
+ * Tested at random a, x.
+ *                a         x                      Relative error:
+ * arithmetic   domain   domain     # trials      peak         rms
+ *    IEEE     0.5,100   0,100      200000       1.9e-14     1.7e-15
+ *    IEEE     0.01,0.5  0,100      200000       1.4e-13     1.6e-15
+ */
+
+/*
+Cephes Math Library Release 2.0:  April, 1987
+Copyright 1985, 1987 by Stephen L. Moshier
+Direct inquiries to 30 Frost Street, Cambridge, MA 02140
+*/
+
+#define MAXLOG 7.09782712893383996843E2
+
+double LMFit::igamc(double a, double x)
+{
+const double big = 1.0 / DBL_EPSILON;
+double ans, ax, c, yc, r, t, y, z;
+double pk, pkm1, pkm2, qk, qkm1, qkm2;
+
+if( (x <= 0) || ( a <= 0) )
+	return( 1.0 );
+
+if( (x < 1.0) || (x < a) )
+	return( 1.0 - igam(a,x) );
+
+ax = a * log(x) - x - lgam(a);
+if( ax < -MAXLOG )
+	{
+	return( 0.0 );
+	}
+ax = exp(ax);
+
+/* continued fraction */
+y = 1.0 - a;
+z = x + y + 1.0;
+c = 0.0;
+pkm2 = 1.0;
+qkm2 = x;
+pkm1 = x + 1.0;
+qkm1 = z * x;
+ans = pkm1/qkm1;
+
+do
+	{
+	c += 1.0;
+	y += 1.0;
+	z += 2.0;
+	yc = y * c;
+	pk = pkm1 * z  -  pkm2 * yc;
+	qk = qkm1 * z  -  qkm2 * yc;
+	if( qk != 0 )
+		{
+		r = pk/qk;
+		t = fabs( (ans - r)/r );
+		ans = r;
+		}
+	else
+		t = 1.0;
+	pkm2 = pkm1;
+	pkm1 = pk;
+	qkm2 = qkm1;
+	qkm1 = qk;
+	if( fabs(pk) > big )
+		{
+		pkm2 *= DBL_EPSILON;
+		pkm1 *= DBL_EPSILON;
+		qkm2 *= DBL_EPSILON;
+		qkm1 *= DBL_EPSILON;
+		}
+	}
+while( t > DBL_EPSILON );
+
+return( ans * ax );
+}
+
+/* left tail of incomplete gamma function:
+ *
+ *          inf.      k
+ *   a  -x   -       x
+ *  x  e     >   ----------
+ *           -     -
+ *          k=0   | (a+k+1)
+ *
+ */
+
+double LMFit::igam(double a, double x)
+{
+double ans, ax, c, r;
+
+if( (x <= 0) || ( a <= 0) )
+	return( 0.0 );
+
+if( (x > 1.0) && (x > a ) )
+	return( 1.0 - igamc(a,x) );
+
+/* Compute  x**a * exp(-x) / gamma(a)  */
+ax = a * log(x) - x - lgam(a);
+if( ax < -MAXLOG )
+	{
+	return( 0.0 );
+	}
+ax = exp(ax);
+
+/* power series */
+r = a;
+c = 1.0;
+ans = 1.0;
+
+do
+	{
+	r += 1.0;
+	c *= x/r;
+	ans += c;
+	}
+while( c/ans > DBL_EPSILON );
+
+return( ans * ax/a );
+}
+
+/* Original function: gsl_sf_lngamma_impl (GNU Scientific Library)
+ * Minor modifications by Werner Stille */
+
+/* Author:  G. Jungman
+ * RCS:     $Id: gamma.c,v 1.25 1998/08/17 00:33:36 jungman Exp $
+ */
+
+#define LogRootTwoPi_  0.9189385332046727418
+
+/* Lanczos with gamma=7, truncated at 1/(z+8)
+ * [J. SIAM Numer. Anal, Ser. B, 1 (1964) 86]
+ */
+double LMFit::lgam(double x)
+{
+/* coefficients for gamma=7, kmax=8  Lanczos method */
+  const double lanczos_7_c[9] = {
+    0.99999999999980993227684700473478,
+    676.520368121885098567009190444019,
+   -1259.13921672240287047156078755283,
+    771.3234287776530788486528258894,
+   -176.61502916214059906584551354,
+    12.507343278686904814458936853,
+   -0.13857109526572011689554707,
+    9.984369578019570859563e-6,
+    1.50563273514931155834e-7
+  };
+
+  int k;
+  double Ag;
+
+  x -= 1.; /* Lanczos writes z! instead of Gamma(z) */
+  Ag = lanczos_7_c[0];
+  for (k = 1; k <= 8; k++)
+    Ag += lanczos_7_c[k] / (x+k);
+  return (x + 0.5) * log(x + 7.5) - (x + 7.5) + LogRootTwoPi_ + log(Ag);
+}
+
+/*							simq.c
+ *
+ *	Solution of simultaneous linear equations AX = B
+ *	by Gaussian elimination with partial pivoting
+ *
+ *
+ *
+ * SYNOPSIS:
+ *
+ * double A[n*n], B[n], X[n];
+ * int n, flag;
+ * int IPS[];
+ * int simq();
+ *
+ * ercode = simq( A, B, X, n, flag, IPS );
+ *
+ *
+ *
+ * DESCRIPTION:
+ *
+ * B, X, IPS are vectors of length n.
+ * A is an n x n matrix (i.e., a vector of length n*n),
+ * stored row-wise: that is, A(i,j) = A[ij],
+ * where ij = i*n + j, which is the transpose of the normal
+ * column-wise storage.
+ *
+ * The contents of matrix A are destroyed.
+ *
+ * Set flag=0 to solve.
+ * Set flag=-1 to do a new back substitution for different B vector
+ * using the same A matrix previously reduced when flag=0.
+ *
+ * The routine returns nonzero on error; messages are printed.
+ *
+ *
+ * ACCURACY:
+ *
+ * Depends on the conditioning (range of eigenvalues) of matrix A.
+ *
+ *
+ * REFERENCE:
+ *
+ * Computer Solution of Linear Algebraic Systems,
+ * by George E. Forsythe and Cleve B. Moler; Prentice-Hall, 1967.
+ *
+ */
+
+/*							simq	2 */
+
+int LMFit::simq(double A[], double B[], double X[], int n, int flag, int IPS[])
+{
+int i, j, ij, ip, ipj, ipk, ipn;
+int idxpiv, iback;
+int k, kp, kp1, kpk, kpn;
+int nip, nkp, nm1;
+double em, q, rownrm, big, size, pivot, sum;
+
+nm1 = n-1;
+if( flag < 0 )
+	goto solve;
+
+/*	Initialize IPS and X	*/
+
+ij=0;
+for( i=0; i<n; i++ )
+	{
+	IPS[i] = i;
+	rownrm = 0.0;
+	for( j=0; j<n; j++ )
+		{
+		q = fabs( A[ij] );
+		if( rownrm < q )
+			rownrm = q;
+		++ij;
+		}
+	if( rownrm == 0.0 )
+		{
+//		puts("SIMQ ROWNRM=0");
+		return(1);
+		}
+	X[i] = 1.0/rownrm;
+	}
+
+/*							simq	3 */
+/*	Gaussian elimination with partial pivoting 	*/
+
+for( k=0; k<nm1; k++ )
+	{
+	big= 0.0;
+	idxpiv = 0;
+	for( i=k; i<n; i++ )
+		{
+		ip = IPS[i];
+		ipk = n*ip + k;
+		size = fabs( A[ipk] ) * X[ip];
+		if( size > big )
+			{
+			big = size;
+			idxpiv = i;
+			}
+		}
+
+	if( big == 0.0 )
+		{
+//		puts( "SIMQ BIG=0" );
+		return(2);
+		}
+	if( idxpiv != k )
+		{
+		j = IPS[k];
+		IPS[k] = IPS[idxpiv];
+		IPS[idxpiv] = j;
+		}
+	kp = IPS[k];
+	kpk = n*kp + k;
+	pivot = A[kpk];
+	kp1 = k+1;
+	for( i=kp1; i<n; i++ )
+		{
+		ip = IPS[i];
+		ipk = n*ip + k;
+		em = -A[ipk]/pivot;
+		A[ipk] = -em;
+		nip = n*ip;
+		nkp = n*kp;
+		for( j=kp1; j<n; j++ )
+			{
+			ipj = nip + j;
+			A[ipj] = A[ipj] + em * A[nkp + j];
+			}
+		}
+	}
+kpn = n * IPS[n-1] + n - 1;	/* last element of IPS[n] th row */
+if( A[kpn] == 0.0 )
+	{
+//	puts( "SIMQ A[kpn]=0");
+	return(3);
+	}
+
+/*							simq 4 */
+/*	back substitution	*/
+
+solve:
+ip = IPS[0];
+X[0] = B[ip];
+for( i=1; i<n; i++ )
+	{
+	ip = IPS[i];
+	ipj = n * ip;
+	sum = 0.0;
+	for( j=0; j<i; j++ )
+		{
+		sum += A[ipj] * X[j];
+		++ipj;
+		}
+	X[i] = B[ip] - sum;
+	}
+
+ipn = n * IPS[n-1] + n - 1;
+X[n-1] = X[n-1]/A[ipn];
+
+for( iback=1; iback<n; iback++ )
+	{
+/* i goes (n-1),...,1	*/
+	i = nm1 - iback;
+	ip = IPS[i];
+	nip = n*ip;
+	sum = 0.0;
+	for( j=i+1; j<n; j++ )
+		sum += A[nip+j] * X[j];
+	X[i] = (X[i] - sum)/A[nip+i];
+	}
+return(0);
+}
+
+/*							mtransp.c
+ *
+ *	Matrix transpose
+ *
+ *
+ *
+ * SYNOPSIS:
+ *
+ * int n;
+ * double A[n*n], T[n*n];
+ *
+ * mtransp( n, A, T );
+ *
+ *
+ *
+ * DESCRIPTION:
+ *
+ *
+ * T[r][c] = A[c][r]
+ *
+ *
+ * Transposes the n by n square matrix A and puts the result in T.
+ * The output, T, may occupy the same storage as A.
+ *
+ *
+ *
+ */
+
+
+void LMFit::mtransp(int n, double* A, double* T)
+{
+int i, j, np1;
+double *pAc, *pAr, *pTc, *pTr, *pA0, *pT0;
+double x;
+
+np1 = n+1;
+pA0 = A;
+pT0 = T;
+for( i=0; i<n-1; i++ ) /* row index */
+	{
+	pAc = pA0; /* next diagonal element of input */
+	pAr = pAc + n; /* next row down underneath the diagonal element */
+	pTc = pT0; /* next diagonal element of the output */
+	pTr = pTc + n; /* next row underneath */
+	*pTc++ = *pAc++; /* copy the diagonal element */
+	for( j=i+1; j<n; j++ ) /* column index */
+		{
+		x = *pAr;
+		*pTr = *pAc++;
+		*pTc++ = x;
+		pAr += n;
+		pTr += n;
+		}
+	pA0 += np1; /* &A[n*i+i] for next i */
+	pT0 += np1; /* &T[n*i+i] for next i */
+	}
+*pT0 = *pA0; /* copy the diagonal element */
+}
+
+void LMFit::fcn(int, int, const double a[], double fvec[], int* iflag)
+{
+  kapp->processEvents();
+  if (userBreak) {
+    *iflag = -1;
+    return;
+  }
+  int j = 0;
+  int i;
+  for (i = 0; i < KPL_NPMAX; i++)
+    if (bFit[i])
+      p[i] = a[j++];
+  int k = 0;
+  for (i = 0; i < (int) ad->count(); i++) {
+    ArrayItem *ar = ad->at(i);
+    double (*f)(double, const double *) = fd->at(i)->fkty;
+    for (j = 0; j < ar->n; j++) {
+      fvec[k] = (ar->x[ar->iy][ar->istart + j] -
+        (*f)(ar->x[ar->ix][ar->istart + j], p)) / sig[k];
+      k++;
+    }
+  }
 }
 
 /************************lmpar.c*************************/
 
 #define BUG 0
 
-void lmpar(int n, double r[], int ldr, const int ipvt[],
-           const double diag[], const double qtb[], double delta,
-           double* par, double x[],
-           double sdiag[], double wa1[], double wa2[])
+void LMFit::lmpar(int n, double r[], int ldr, const int ipvt[],
+                  const double diag[], const double qtb[], double delta,
+                  double* par, double x[],
+                  double sdiag[], double wa1[], double wa2[])
 {
 /*     **********
 *
@@ -1014,7 +1546,7 @@ for( j=0; j<n; j++ )
 gnorm = enorm(n,wa1);
 paru = gnorm/delta;
 if(paru == zero)
-	paru = DWARF/QMIN(delta,p1);
+	paru = DBL_MIN/QMIN(delta,p1);
 /*
 *     if the input par lies outside of the interval (parl,paru),
 *     set par to the closer endpoint.
@@ -1035,7 +1567,7 @@ iter += 1;
 *	 evaluate the function at the current value of par.
 */
 if( *par == zero)
-	*par = QMAX(DWARF,p001*paru);
+	*par = QMAX(DBL_MIN,p001*paru);
 temp = sqrt( *par );
 for( j=0; j<n; j++ )
 	wa1[j] = temp*diag[j];
@@ -1108,12 +1640,131 @@ if(iter == 0)
 */
 }
 
+/************************fdjac2.c*************************/
+
+#define BUG 0
+
+void LMFit::fdjac2(int m, int n, double x[], const double fvec[], double fjac[],
+                   int, int *iflag, double epsfcn, double wa[])
+{
+/*
+*     **********
+*
+*     subroutine fdjac2
+*
+*     this subroutine computes a forward-difference approximation
+*     to the m by n jacobian matrix associated with a specified
+*     problem of m functions in n variables.
+*
+*     the subroutine statement is
+*
+*	subroutine fdjac2(fcn,m,n,x,fvec,fjac,ldfjac,iflag,epsfcn,wa)
+*
+*     where
+*
+*	fcn is the name of the user-supplied subroutine which
+*	  calculates the functions. fcn must be declared
+*	  in an external statement in the user calling
+*	  program, and should be written as follows.
+*
+*	  subroutine fcn(m,n,x,fvec,iflag)
+*	  integer m,n,iflag
+*	  double precision x(n),fvec(m)
+*	  ----------
+*	  calculate the functions at x and
+*	  return this vector in fvec.
+*	  ----------
+*	  return
+*	  end
+*
+*	  the value of iflag should not be changed by fcn unless
+*	  the user wants to terminate execution of fdjac2.
+*	  in this case set iflag to a negative integer.
+*
+*	m is a positive integer input variable set to the number
+*	  of functions.
+*
+*	n is a positive integer input variable set to the number
+*	  of variables. n must not exceed m.
+*
+*	x is an input array of length n.
+*
+*	fvec is an input array of length m which must contain the
+*	  functions evaluated at x.
+*
+*	fjac is an output m by n array which contains the
+*	  approximation to the jacobian matrix evaluated at x.
+*
+*	ldfjac is a positive integer input variable not less than m
+*	  which specifies the leading dimension of the array fjac.
+*
+*	iflag is an integer variable which can be used to terminate
+*	  the execution of fdjac2. see description of fcn.
+*
+*	epsfcn is an input variable used in determining a suitable
+*	  step length for the forward-difference approximation. this
+*	  approximation assumes that the relative errors in the
+*	  functions are of the order of epsfcn. if epsfcn is less
+*	  than the machine precision, it is assumed that the relative
+*	  errors in the functions are of the order of the machine
+*	  precision.
+*
+*	wa is a work array of length m.
+*
+*     subprograms called
+*
+*	user-supplied ...... fcn
+*
+*	minpack-supplied ... dpmpar
+*
+*	fortran-supplied ... dabs,dmax1,dsqrt
+*
+*     argonne national laboratory. minpack project. march 1980.
+*     burton s. garbow, kenneth e. hillstrom, jorge j. more
+*
+      **********
+*/
+int i,j,ij;
+double eps,h,temp;
+static double zero = 0.0;
+
+temp = QMAX(epsfcn,DBL_EPSILON);
+eps = sqrt(temp);
+#if BUG
+printf( "fdjac2\n" );
+#endif
+ij = 0;
+for( j=0; j<n; j++ )
+	{
+	temp = x[j];
+	h = eps * fabs(temp);
+	if(h == zero)
+		h = eps;
+	x[j] = temp + h;
+	fcn(m,n,x,wa,iflag);
+	if( *iflag < 0)
+		return;
+	x[j] = temp;
+	for( i=0; i<m; i++ )
+		{
+		fjac[ij] = (wa[i] - fvec[i])/h;
+		ij += 1;	/* fjac[i+m*j] */
+		}
+	}
+#if BUG
+pmat( m, n, fjac );
+#endif
+/*
+*     last card of subroutine fdjac2.
+*/
+}
+
 /************************qrfac.c*************************/
 
 #define BUG 0
 
-void qrfac(int m, int n, double a[], int, int pivot, int ipvt[],
-           int, double rdiag[], double acnorm[], double wa[])
+void LMFit::qrfac(int m, int n, double a[], int, int pivot, int ipvt[],
+                  int, double rdiag[], double acnorm[], double wa[])
 {
 /*
 *     **********
@@ -1300,7 +1951,7 @@ for( k=jp1; k<n; k++ )
 		temp = QMAX( zero, one-temp*temp );
 		rdiag[k] *= sqrt(temp);
 		temp = rdiag[k]/wa[k];
-		if( (p05*temp*temp) <= MACHEP)
+		if( (p05*temp*temp) <= DBL_EPSILON)
 			{
 			rdiag[k] = enorm(m-j-1,&a[jp1+m*k]);
 			wa[k] = rdiag[k];
@@ -1321,9 +1972,9 @@ L100:
 
 #define BUG 0
 
-void qrsolv(int n, double r[], int ldr, const int ipvt[],
-            const double diag[], const double qtb[], double x[],
-            double sdiag[], double wa[])
+void LMFit::qrsolv(int n, double r[], int ldr, const int ipvt[],
+                   const double diag[], const double qtb[], double x[],
+                   double sdiag[], double wa[])
 {
 /*
 *     **********
@@ -1405,7 +2056,7 @@ void qrsolv(int n, double r[], int ldr, const int ipvt[],
 *     **********
 */
 int i,ij,ik,kk,j,jp1,k,kp1,l,nsing;
-double cos,cotan,qtbpj,sin,sum,tan,temp;
+double cos_v,cotan,qtbpj,sin_v,sum,tan_v,temp;
 static double zero = 0.0;
 static double p25 = 0.25;
 static double p5 = 0.5;
@@ -1465,22 +2116,22 @@ for( k=j; k<n; k++ )
 	if(fabs(r[kk]) < fabs(sdiag[k]))
 		{
 		cotan = r[kk]/sdiag[k];
-		sin = p5/sqrt(p25+p25*cotan*cotan);
-		cos = sin*cotan;
+		sin_v = p5/sqrt(p25+p25*cotan*cotan);
+		cos_v = sin_v*cotan;
 		}
 	else
 		{
-		tan = sdiag[k]/r[kk];
-		cos = p5/sqrt(p25+p25*tan*tan);
-		sin = cos*tan;
+		tan_v = sdiag[k]/r[kk];
+		cos_v = p5/sqrt(p25+p25*tan_v*tan_v);
+		sin_v = cos_v*tan_v;
 		}
 /*
 *	    compute the modified diagonal element of r and
 *	    the modified element of ((q transpose)*b,0).
 */
-	r[kk] = cos*r[kk] + sin*sdiag[k];
-	temp = cos*wa[k] + sin*qtbpj;
-	qtbpj = -sin*wa[k] + cos*qtbpj;
+	r[kk] = cos_v*r[kk] + sin_v*sdiag[k];
+	temp = cos_v*wa[k] + sin_v*qtbpj;
+	qtbpj = -sin_v*wa[k] + cos_v*qtbpj;
 	wa[k] = temp;
 /*
 *	    accumulate the tranformation in the row of s.
@@ -1491,8 +2142,8 @@ for( k=j; k<n; k++ )
 		ik = kk + 1;
 		for( i=kp1; i<n; i++ )
 			{
-			temp = cos*r[ik] + sin*sdiag[i];
-			sdiag[i] = -sin*r[ik] + cos*sdiag[i];
+			temp = cos_v*r[ik] + sin_v*sdiag[i];
+			sdiag[i] = -sin_v*r[ik] + cos_v*sdiag[i];
 			r[ik] = temp;
 			ik += 1; /* [i+ldr*k] */
 			}
@@ -1551,661 +2202,3 @@ for( j=0; j<n; j++ )
 *     last card of subroutine qrsolv.
 */
 }
-
-/************************enorm.c*************************/
-
-double enorm(int n, const double x[])
-{
-/*
-*     **********
-*
-*     function enorm
-*
-*     given an n-vector x, this function calculates the
-*     euclidean norm of x.
-*
-*     the euclidean norm is computed by accumulating the sum of
-*     squares in three different sums. the sums of squares for the
-*     small and large components are scaled so that no overflows
-*     occur. non-destructive underflows are permitted. underflows
-*     and overflows do not occur in the computation of the unscaled
-*     sum of squares for the intermediate components.
-*     the definitions of small, intermediate and large components
-*     depend on two constants, rdwarf and rgiant. the main
-*     restrictions on these constants are that rdwarf**2 not
-*     underflow and rgiant**2 not overflow. the constants
-*     given here are suitable for every known computer.
-*
-*     the function statement is
-*
-*	double precision function enorm(n,x)
-*
-*     where
-*
-*	n is a positive integer input variable.
-*
-*	x is an input array of length n.
-*
-*     subprograms called
-*
-*	fortran-supplied ... dabs,dsqrt
-*
-*     argonne national laboratory. minpack project. march 1980.
-*     burton s. garbow, kenneth e. hillstrom, jorge j. more
-*
-*     **********
-*/
-int i;
-double agiant,floatn,s1,s2,s3,xabs,x1max,x3max;
-double ans, temp;
-static double rdwarf = 3.834e-20;
-static double rgiant = 1.304e19;
-static double zero = 0.0;
-static double one = 1.0;
-
-s1 = zero;
-s2 = zero;
-s3 = zero;
-x1max = zero;
-x3max = zero;
-floatn = n;
-agiant = rgiant/floatn;
-
-for( i=0; i<n; i++ )
-{
-xabs = fabs(x[i]);
-if( (xabs > rdwarf) && (xabs < agiant) )
-	{
-/*
-*	    sum for intermediate components.
-*/
-	s2 += xabs*xabs;
-	continue;
-	}
-
-if(xabs > rdwarf)
-	{
-/*
-*	       sum for large components.
-*/
-	if(xabs > x1max)
-		{
-		temp = x1max/xabs;
-		s1 = one + s1*temp*temp;
-		x1max = xabs;
-		}
-	else
-		{
-		temp = xabs/x1max;
-		s1 += temp*temp;
-		}
-	continue;
-	}
-/*
-*	       sum for small components.
-*/
-if(xabs > x3max)
-	{
-	temp = x3max/xabs;
-	s3 = one + s3*temp*temp;
-	x3max = xabs;
-	}
-else	
-	{
-	if(xabs != zero)
-		{
-		temp = xabs/x3max;
-		s3 += temp*temp;
-		}
-	}
-}
-/*
-*     calculation of norm.
-*/
-if(s1 != zero)
-	{
-	temp = s1 + (s2/x1max)/x1max;
-	ans = x1max*sqrt(temp);
-	return(ans);
-	}
-if(s2 != zero)
-	{
-	if(s2 >= x3max)
-		temp = s2*(one+(x3max/s2)*(x3max*s3));
-	else
-		temp = x3max*((s2/x3max)+(x3max*s3));
-	ans = sqrt(temp);
-	}
-else
-	{
-	ans = x3max*sqrt(s3);
-	}
-return(ans);
-/*
-*     last card of function enorm.
-*/
-}
-
-/*							minv.c
- *
- *	Matrix inversion
- *
- *
- *
- * SYNOPSIS:
- *
- * int n, errcod;
- * double A[n*n], X[n*n];
- * double B[n];
- * int IPS[n];
- * int minv();
- *
- * errcod = minv( A, X, n, B, IPS );
- *
- *
- *
- * DESCRIPTION:
- *
- * Finds the inverse of the n by n matrix A.  The result goes
- * to X.   B and IPS are scratch pad arrays of length n.
- * The contents of matrix A are destroyed.
- *
- * The routine returns nonzero on error; error messages are printed
- * by subroutine simq().
- *
- */
-
-int minv(double A[], double X[], int n, double B[], int IPS[])
-{
-double *pX;
-int i, k;
-
-for( i=1; i<n; i++ )
-	B[i] = 0.0;
-B[0] = 1.0;
-/* Reduce the matrix and solve for first right hand side vector */
-pX = X;
-k = simq( A, B, pX, n, 1, IPS );
-if( k )
-	return(-1);
-/* Solve for the remaining right hand side vectors */
-for( i=1; i<n; i++ )
-	{
-	B[i-1] = 0.0;
-	B[i] = 1.0;
-	pX += n;
-	k = simq( A, B, pX, n, -1, IPS );
-	if( k )
-		return(-1);
-	}
-/* Transpose the array of solution vectors */
-mtransp( n, X, X );
-return(0);
-}
-
-/*							igam.c
- *
- *	Incomplete gamma integral
- *
- *
- *
- * SYNOPSIS:
- *
- * double a, x, y, igam();
- *
- * y = igam( a, x );
- *
- * DESCRIPTION:
- *
- * The function is defined by
- *
- *                           x
- *                            -
- *                   1       | |  -t  a-1
- *  igam(a,x)  =   -----     |   e   t   dt.
- *                  -      | |
- *                 | (a)    -
- *                           0
- *
- *
- * In this implementation both arguments must be positive.
- * The integral is evaluated by either a power series or
- * continued fraction expansion, depending on the relative
- * values of a and x.
- *
- * ACCURACY:
- *
- *                      Relative error:
- * arithmetic   domain     # trials      peak         rms
- *    IEEE      0,30       200000       3.6e-14     2.9e-15
- *    IEEE      0,100      300000       9.9e-14     1.5e-14
- */
-/*							igamc()
- *
- *	Complemented incomplete gamma integral
- *
- *
- *
- * SYNOPSIS:
- *
- * double a, x, y, igamc();
- *
- * y = igamc( a, x );
- *
- * DESCRIPTION:
- *
- * The function is defined by
- *
- *
- *  igamc(a,x)   =   1 - igam(a,x)
- *
- *                            inf.
- *                              -
- *                     1       | |  -t  a-1
- *               =   -----     |   e   t   dt.
- *                    -      | |
- *                   | (a)    -
- *                             x
- *
- *
- * In this implementation both arguments must be positive.
- * The integral is evaluated by either a power series or
- * continued fraction expansion, depending on the relative
- * values of a and x.
- *
- * ACCURACY:
- *
- * Tested at random a, x.
- *                a         x                      Relative error:
- * arithmetic   domain   domain     # trials      peak         rms
- *    IEEE     0.5,100   0,100      200000       1.9e-14     1.7e-15
- *    IEEE     0.01,0.5  0,100      200000       1.4e-13     1.6e-15
- */
-
-/*
-Cephes Math Library Release 2.0:  April, 1987
-Copyright 1985, 1987 by Stephen L. Moshier
-Direct inquiries to 30 Frost Street, Cambridge, MA 02140
-*/
-
-#define MACHEP DBL_EPSILON
-#define MAXLOG 7.09782712893383996843E2
-
-const double big = 4.503599627370496e15;
-const double biginv =  2.22044604925031308085e-16;
-
-double igamc(double a, double x)
-{
-double ans, ax, c, yc, r, t, y, z;
-double pk, pkm1, pkm2, qk, qkm1, qkm2;
-
-if( (x <= 0) || ( a <= 0) )
-	return( 1.0 );
-
-if( (x < 1.0) || (x < a) )
-	return( 1.0 - igam(a,x) );
-
-ax = a * log(x) - x - lgam(a);
-if( ax < -MAXLOG )
-	{
-	return( 0.0 );
-	}
-ax = exp(ax);
-
-/* continued fraction */
-y = 1.0 - a;
-z = x + y + 1.0;
-c = 0.0;
-pkm2 = 1.0;
-qkm2 = x;
-pkm1 = x + 1.0;
-qkm1 = z * x;
-ans = pkm1/qkm1;
-
-do
-	{
-	c += 1.0;
-	y += 1.0;
-	z += 2.0;
-	yc = y * c;
-	pk = pkm1 * z  -  pkm2 * yc;
-	qk = qkm1 * z  -  qkm2 * yc;
-	if( qk != 0 )
-		{
-		r = pk/qk;
-		t = fabs( (ans - r)/r );
-		ans = r;
-		}
-	else
-		t = 1.0;
-	pkm2 = pkm1;
-	pkm1 = pk;
-	qkm2 = qkm1;
-	qkm1 = qk;
-	if( fabs(pk) > big )
-		{
-		pkm2 *= biginv;
-		pkm1 *= biginv;
-		qkm2 *= biginv;
-		qkm1 *= biginv;
-		}
-	}
-while( t > MACHEP );
-
-return( ans * ax );
-}
-
-/* left tail of incomplete gamma function:
- *
- *          inf.      k
- *   a  -x   -       x
- *  x  e     >   ----------
- *           -     -
- *          k=0   | (a+k+1)
- *
- */
-
-double igam(double a, double x)
-{
-double ans, ax, c, r;
-
-if( (x <= 0) || ( a <= 0) )
-	return( 0.0 );
-
-if( (x > 1.0) && (x > a ) )
-	return( 1.0 - igamc(a,x) );
-
-/* Compute  x**a * exp(-x) / gamma(a)  */
-ax = a * log(x) - x - lgam(a);
-if( ax < -MAXLOG )
-	{
-	return( 0.0 );
-	}
-ax = exp(ax);
-
-/* power series */
-r = a;
-c = 1.0;
-ans = 1.0;
-
-do
-	{
-	r += 1.0;
-	c *= x/r;
-	ans += c;
-	}
-while( c/ans > MACHEP );
-
-return( ans * ax/a );
-}
-
-/* Original function: gsl_sf_lngamma_impl (GNU Scientific Library)
- * Minor modifications by Werner Stille */
-
-/* Author:  G. Jungman
- * RCS:     $Id: gamma.c,v 1.25 1998/08/17 00:33:36 jungman Exp $
- */
-
-#define LogRootTwoPi_  0.9189385332046727418
-
-/* coefficients for gamma=7, kmax=8  Lanczos method */
-const double lanczos_7_c[9] = {
-  0.99999999999980993227684700473478,
-  676.520368121885098567009190444019,
- -1259.13921672240287047156078755283,
-  771.3234287776530788486528258894,
- -176.61502916214059906584551354,
-  12.507343278686904814458936853,
- -0.13857109526572011689554707,
-  9.984369578019570859563e-6,
-  1.50563273514931155834e-7
-};
-
-/* Lanczos with gamma=7, truncated at 1/(z+8)
- * [J. SIAM Numer. Anal, Ser. B, 1 (1964) 86]
- */
-double lgam(double x)
-{
-  int k;
-  double Ag;
-
-  x -= 1.; /* Lanczos writes z! instead of Gamma(z) */
-  Ag = lanczos_7_c[0];
-  for (k = 1; k <= 8; k++)
-    Ag += lanczos_7_c[k] / (x+k);
-  return (x + 0.5) * log(x + 7.5) - (x + 7.5) + LogRootTwoPi_ + log(Ag);
-}
-
-/*							simq.c
- *
- *	Solution of simultaneous linear equations AX = B
- *	by Gaussian elimination with partial pivoting
- *
- *
- *
- * SYNOPSIS:
- *
- * double A[n*n], B[n], X[n];
- * int n, flag;
- * int IPS[];
- * int simq();
- *
- * ercode = simq( A, B, X, n, flag, IPS );
- *
- *
- *
- * DESCRIPTION:
- *
- * B, X, IPS are vectors of length n.
- * A is an n x n matrix (i.e., a vector of length n*n),
- * stored row-wise: that is, A(i,j) = A[ij],
- * where ij = i*n + j, which is the transpose of the normal
- * column-wise storage.
- *
- * The contents of matrix A are destroyed.
- *
- * Set flag=0 to solve.
- * Set flag=-1 to do a new back substitution for different B vector
- * using the same A matrix previously reduced when flag=0.
- *
- * The routine returns nonzero on error; messages are printed.
- *
- *
- * ACCURACY:
- *
- * Depends on the conditioning (range of eigenvalues) of matrix A.
- *
- *
- * REFERENCE:
- *
- * Computer Solution of Linear Algebraic Systems,
- * by George E. Forsythe and Cleve B. Moler; Prentice-Hall, 1967.
- *
- */
-
-/*							simq	2 */
-
-int simq(double A[], double B[], double X[], int n, int flag, int IPS[])
-{
-int i, j, ij, ip, ipj, ipk, ipn;
-int idxpiv, iback;
-int k, kp, kp1, kpk, kpn;
-int nip, nkp, nm1;
-double em, q, rownrm, big, size, pivot, sum;
-
-nm1 = n-1;
-if( flag < 0 )
-	goto solve;
-
-/*	Initialize IPS and X	*/
-
-ij=0;
-for( i=0; i<n; i++ )
-	{
-	IPS[i] = i;
-	rownrm = 0.0;
-	for( j=0; j<n; j++ )
-		{
-		q = fabs( A[ij] );
-		if( rownrm < q )
-			rownrm = q;
-		++ij;
-		}
-	if( rownrm == 0.0 )
-		{
-//		puts("SIMQ ROWNRM=0");
-		return(1);
-		}
-	X[i] = 1.0/rownrm;
-	}
-
-/*							simq	3 */
-/*	Gaussian elimination with partial pivoting 	*/
-
-for( k=0; k<nm1; k++ )
-	{
-	big= 0.0;
-	idxpiv = 0;
-	for( i=k; i<n; i++ )
-		{
-		ip = IPS[i];
-		ipk = n*ip + k;
-		size = fabs( A[ipk] ) * X[ip];
-		if( size > big )
-			{
-			big = size;
-			idxpiv = i;
-			}
-		}
-
-	if( big == 0.0 )
-		{
-//		puts( "SIMQ BIG=0" );
-		return(2);
-		}
-	if( idxpiv != k )
-		{
-		j = IPS[k];
-		IPS[k] = IPS[idxpiv];
-		IPS[idxpiv] = j;
-		}
-	kp = IPS[k];
-	kpk = n*kp + k;
-	pivot = A[kpk];
-	kp1 = k+1;
-	for( i=kp1; i<n; i++ )
-		{
-		ip = IPS[i];
-		ipk = n*ip + k;
-		em = -A[ipk]/pivot;
-		A[ipk] = -em;
-		nip = n*ip;
-		nkp = n*kp;
-		for( j=kp1; j<n; j++ )
-			{
-			ipj = nip + j;
-			A[ipj] = A[ipj] + em * A[nkp + j];
-			}
-		}
-	}
-kpn = n * IPS[n-1] + n - 1;	/* last element of IPS[n] th row */
-if( A[kpn] == 0.0 )
-	{
-//	puts( "SIMQ A[kpn]=0");
-	return(3);
-	}
-
-/*							simq 4 */
-/*	back substitution	*/
-
-solve:
-ip = IPS[0];
-X[0] = B[ip];
-for( i=1; i<n; i++ )
-	{
-	ip = IPS[i];
-	ipj = n * ip;
-	sum = 0.0;
-	for( j=0; j<i; j++ )
-		{
-		sum += A[ipj] * X[j];
-		++ipj;
-		}
-	X[i] = B[ip] - sum;
-	}
-
-ipn = n * IPS[n-1] + n - 1;
-X[n-1] = X[n-1]/A[ipn];
-
-for( iback=1; iback<n; iback++ )
-	{
-/* i goes (n-1),...,1	*/
-	i = nm1 - iback;
-	ip = IPS[i];
-	nip = n*ip;
-	sum = 0.0;
-	for( j=i+1; j<n; j++ )
-		sum += A[nip+j] * X[j];
-	X[i] = (X[i] - sum)/A[nip+i];
-	}
-return(0);
-}
-
-/*							mtransp.c
- *
- *	Matrix transpose
- *
- *
- *
- * SYNOPSIS:
- *
- * int n;
- * double A[n*n], T[n*n];
- *
- * mtransp( n, A, T );
- *
- *
- *
- * DESCRIPTION:
- *
- *
- * T[r][c] = A[c][r]
- *
- *
- * Transposes the n by n square matrix A and puts the result in T.
- * The output, T, may occupy the same storage as A.
- *
- *
- *
- */
-
-
-void mtransp(int n, double* A, double* T)
-{
-int i, j, np1;
-double *pAc, *pAr, *pTc, *pTr, *pA0, *pT0;
-double x;
-
-np1 = n+1;
-pA0 = A;
-pT0 = T;
-for( i=0; i<n-1; i++ ) /* row index */
-	{
-	pAc = pA0; /* next diagonal element of input */
-	pAr = pAc + n; /* next row down underneath the diagonal element */
-	pTc = pT0; /* next diagonal element of the output */
-	pTr = pTc + n; /* next row underneath */
-	*pTc++ = *pAc++; /* copy the diagonal element */
-	for( j=i+1; j<n; j++ ) /* column index */
-		{
-		x = *pAr;
-		*pTr = *pAc++;
-		*pTc++ = x;
-		pAr += n;
-		pTr += n;
-		}
-	pA0 += np1; /* &A[n*i+i] for next i */
-	pT0 += np1; /* &T[n*i+i] for next i */
-	}
-*pT0 = *pA0; /* copy the diagonal element */
-}
