@@ -6,7 +6,7 @@
     data sets and functions.
 
     begin                : Wed Sep 08 1999
-    copyright            : (C) 2001 by Werner Stille
+    copyright            : (C) 2002 by Werner Stille
     email                : stille@uni-freiburg.de
  ***************************************************************************/
 
@@ -24,7 +24,6 @@
 #include <qdatetime.h>
 #include <math.h>
 #include "psgraph.h"
-#include "utils.h"
 
 const double PSGraph::pts = 28.34645669;
 
@@ -54,53 +53,8 @@ void PSGraph::psInit(QTextStream* t, double xf, double yf)
     "%%Creator: Kpl\n"
     "%%CreationDate: " << QDateTime::currentDateTime().toString() << "\n"
     "%%Pages: 1\n\n"
-    "%User code starts at line 444\n\n"
-    << "/enc [\n 16#A0 /space\n 16#A4 /currency\n"
-    " 16#A6 /brokenbar\n 16#A8 /dieresis\n"
-    " 16#A9 /copyright\n 16#AA /ordfeminine\n"
-    " 16#AC /logicalnot\n 16#AD /endash\n"
-    " 16#AE /registered\n 16#AF /macron\n"
-    " 16#B0 /ring\n 16#B1 /plusminus\n"
-    " 16#B2 /twosuperior\n 16#B3 /threesuperior\n"
-    " 16#B4 /acute\n 16#B5 /mu\n"
-    " 16#B7 /periodcentered\n 16#B8 /cedilla\n"
-    " 16#B9 /onesuperior\n 16#BA /ordmasculine\n"
-    " 16#BC /onequarter\n 16#BD /onehalf\n"
-    " 16#BE /threequarters\n 16#C0 /Agrave\n"
-    " 16#C1 /Aacute\n 16#C2 /Acircumflex\n"
-    " 16#C3 /Atilde\n 16#C4 /Adieresis\n"
-    " 16#C5 /Aring\n 16#C6 /AE\n"
-    " 16#C7 /Ccedilla\n 16#C8 /Egrave\n"
-    " 16#C9 /Eacute\n 16#CA /Ecircumflex\n"
-    " 16#CB /Edieresis\n 16#CC /Igrave\n"
-    " 16#CD /Iacute\n 16#CE /Icircumflex\n"
-    " 16#CF /Idieresis\n 16#D0 /Eth\n"
-    " 16#D1 /Ntilde\n 16#D2 /Ograve\n"
-    " 16#D3 /Oacute\n 16#D4 /Ocircumflex\n"
-    " 16#D5 /Otilde\n 16#D6 /Odieresis\n"
-    " 16#D7 /multiply\n 16#D8 /Oslash\n"
-    " 16#D9 /Ugrave\n 16#DA /Uacute\n"
-    " 16#DB /Ucircumflex\n 16#DC /Udieresis\n"
-    " 16#DD /Yacute\n 16#DE /Thorn\n"
-    " 16#DF /germandbls\n 16#E0 /agrave\n"
-    " 16#E1 /aacute\n 16#E2 /acircumflex\n"
-    " 16#E3 /atilde\n 16#E4 /adieresis\n"
-    " 16#E5 /aring\n 16#E6 /ae\n"
-    " 16#E7 /ccedilla\n 16#E8 /egrave\n"
-    " 16#E9 /eacute\n 16#EA /ecircumflex\n"
-    " 16#EB /edieresis\n 16#EC /igrave\n"
-    " 16#ED /iacute\n 16#EE /icircumflex\n"
-    " 16#EF /idieresis\n 16#F0 /eth\n"
-    " 16#F1 /ntilde\n 16#F2 /ograve\n"
-    " 16#F3 /oacute\n 16#F4 /ocircumflex\n"
-    " 16#F5 /otilde\n 16#F6 /odieresis\n"
-    " 16#F7 /divide\n 16#F8 /oslash\n"
-    " 16#F9 /ugrave\n 16#FA /uacute\n"
-    " 16#FB /ucircumflex\n 16#FC /udieresis\n"
-    " 16#FD /yacute\n 16#FE /thorn\n"
-    " 16#FF /ydieresis\n] def\n"
-    << "/ws 12 dict def\n"
-    "/xmin 0 def\n"
+    "%User code starts at line 392\n\n"
+    << "/xmin 0 def\n"
     "/ymin 0 def\n"
     "/xmax 21 def\n"
     "/ymax 29.7 def\n"
@@ -120,20 +74,15 @@ void PSGraph::psInit(QTextStream* t, double xf, double yf)
     "/symbol 0 def\n"
     "/dpos 0 def\n"
     "/relSize 1 def\n"
+    "/ndigx -1 def\n"
+    "/ndigy -1 def\n"
     "/logx false def\n"
     "/logy false def\n\n"
     << "/ed {exch def} bind def\n\n"
     << "/encfont {\n"
-    " ws begin\n /newname ed /orgname ed /orgdict orgname findfont def\n"
-    " /newfont orgdict maxlength dict def orgdict {\n"
-    "  exch dup /FontID ne {\n"
-    "   dup /Encoding eq {\n"
-    "    exch dup length array copy newfont 3 1 roll put\n"
-    "   } {exch newfont 3 1 roll put} ifelse\n"
-    "  } {pop pop} ifelse\n"
-    " } forall newfont /FontName newname put enc aload pop\n"
-    " enc length 2 idiv {newfont /Encoding get 3 1 roll put} repeat\n"
-    " newname newfont definefont end\n"
+    " findfont dup length dict begin {\n"
+    "  1 index dup /FID eq exch /Encoding eq or {pop pop} {def} ifelse\n"
+    " } forall /Encoding ISOLatin1Encoding def currentdict end definefont\n"
     "} bind def\n\n"
     << "/strcat {\n"
     " /s2 ed /s1 ed s1 length dup s2 length add string /s3 ed s3 exch\n"
@@ -176,13 +125,13 @@ void PSGraph::psInit(QTextStream* t, double xf, double yf)
     " f mul p1 0 eq {put dup dup 0 get 2 exch} {0.5 mul put dup 2 p1 f mul}\n"
     " ifelse put dup dup 1 get 3 exch put dpos setdash\n"
     "} bind def\n\n"
-    << "/err {\n fy mul yscal mul /dy ed fy mul /y ed logy\n"
+    << "/err {\n fy mul yscal mul /dy ed fy mul /y ed fx mul /x ed logy\n"
     " {y 0 gt {/dy dy y div 0.4343 mul def} {/dy 0 def} ifelse} if\n"
-    " dy sh lt {pop} {\n"
-    "  fx mul logx {log} if y logy {log} if mv 0 sh rmoveto 0 dy sh sub\n"
+    " x 0 gt logx not or dy sh ge and {\n"
+    "  x logx {log} if y logy {log} if mv 0 sh rmoveto 0 dy sh sub\n"
     "  rlineto 0.5 sh mul 0 rmoveto sh neg 0 rlineto 0 -2 dy mul rmoveto sh\n"
     "  0 rlineto -0.5 sh mul 0 rmoveto 0 dy sh sub rlineto stroke\n"
-    " } ifelse\n"
+    " } if\n"
     "} bind def\n\n"
     << "/format {\n /ymax ed /xmax ed /xmin 0 def /ymin 0 def\n"
     " /xscal scalx def /yscal xscal def /minx 0 def /miny 0 def /maxx xscal\n"
@@ -212,88 +161,119 @@ void PSGraph::psInit(QTextStream* t, double xf, double yf)
     " rect clip\n"
     "} bind def\n\n"
     << "/prenum {\n"
-    " /ndig ed ndig 1 lt\n"
-    " {round ndig 0 eq\n"
-    "  {cvr 256 string cvs dup 0 exch length 1 sub getinterval}\n"
-    "  {cvi 256 string cvs} ifelse}\n"
-    " {/f 10 ndig exp def f mul round f div cvr 256 string cvs dup (.)\n"
-    " search {pop pop length ndig exch sub /n ed n 0 ne\n"
-    " {dup length n add string /t ed t 0 3 -1 roll putinterval\n"
-    " 1 1 n {t length exch sub t exch 48 put} for t}\n"
-    " if} {pop} ifelse} ifelse\n"
+    " /ndig ed /v ed v 0 lt {/v v neg def (-)} {()} ifelse /v ndig 0 gt\n"
+    " {10 ndig exp dup v mul round dup cvi /iv ed exch div}\n"
+    " {v round dup cvi /iv ed} ifelse def /ie v 10 lt {0} {v log cvi}\n"
+    " ifelse def /iend ndig 0 gt {ndig neg} {0} ifelse def\n"
+    " /id 10 ie iend sub exp round cvi def {\n"
+    "  iv dup id mod 10 mul /iv ed id idiv 256 string cvs\n"
+    "  strcat ie 0 eq {ndig 0 ge {(.) strcat} if} if ie iend eq\n"
+    "  {exit} if /ie ie 1 sub def\n"
+    " } loop\n"
     "} bind def\n\n"
     << "/incLog {\n"
-    " log /lgf ed /xlg ed /mant xlg dup dup abs 1.0e-6 mul add cvi sub def\n"
-    " mant abs 1.0e-9 lt {/mant 0 def} if mant 0 lt {/mant mant 1 add def}\n"
+    " log /lgf ed /xlg ed /mant xlg dup dup abs 1.0e-5 mul add cvi sub def\n"
+    " mant abs 1.0e-6 lt {/mant 0 def} if mant 0 lt {/mant mant 1 add def}\n"
     " if xlg mant 0.1 lt {lgf add} {mant 0.4 lt {2.5 log add}\n"
     "  {mant 0.6 lt {10 3 div log add} {2 log add} ifelse} ifelse} ifelse\n"
     "} bind def\n\n"
     << "/raster {\n"
-    " /yRight ed /xTop ed /mode ed /inty ed /intx ed /ytic ed /xtic ed\n"
-    " mode 0 ne {\n"
+    " /yRight ed /yLeft ed /xTop ed /xBottom ed /yOff ed /xOff ed /mode ed\n"
+    " /inty ed /intx ed /ytic ed /xtic ed mode -1 ne {mode 0 ne {\n"
     "  logx {xtic 2.5 lt {/xtic 2 def} {xtic 5.5 lt {/xtic 3 def}\n"
-    "   {xtic 99 lt {/xtic 10 def} if}} ifelse} if\n"
+    "   {xtic 99 lt {/xtic 10 def} if} ifelse} ifelse} if\n"
+    "  logy {ytic 2.5 lt {/ytic 2 def} {ytic 5.5 lt {/ytic 3 def}\n"
+    "   {ytic 99 lt {/ytic 10 def} if} ifelse} ifelse} if\n"
     "  colgrid scol 0.00208 slw /dx maxx minx sub def /dy maxy miny sub def\n"
-    "  mode 2 le {/dx dx 0.02 mul def /dy dx def} if /x xmin def {logx\n"
+    "  mode 2 le {/dx dx 0.02 mul def /dy dx def} if\n"
+    "  mode 2 le intx 1 gt and xOff 0 gt and logx not and {\n"
+    "   /d xtic intx div def /xm xmin xOff add d sub def {xm xmin le {exit} if\n"
+    "    xBottom {xm ymin mv 0 dy 0.5 mul rlineto stroke} if\n"
+    "    xTop {xm ymax mv 0 dy -0.5 mul rlineto stroke} if\n"
+    "    /xm xm d sub def} loop} if\n"
+    "  /x xmin xOff add def {logx\n"
     "   {/xa x def /x x xtic incLog def\n"
-    "    x xmax lt {x xTop {ymax} {ymin} ifelse mv 0 dy xTop {neg} if\n"
-    "    rlineto stroke} if intx 1 gt {\n"
+    "    x xmax lt {xBottom {x ymin mv 0 dy rlineto stroke} if\n"
+    "    xTop {x ymax mv 0 dy neg rlineto stroke} if} if intx 1 gt {\n"
     "     /xd 10 xa mant sub exp def {/xa 10 xa exp xd add log def\n"
-    "      xa x ge xa xmax ge or {exit} if xa xTop {ymax} {ymin} ifelse\n"
-    "      mv 0 dy xTop {neg} if\n"
-    "      mode 2 le {0.5 mul} if rlineto stroke} loop\n"
+    "      xa x ge xa xmax ge or {exit} if\n"
+    "      xBottom {xa ymin mv 0 dy mode 2 le {0.5 mul} if rlineto stroke} if\n"
+    "      xTop {xa ymax mv 0 dy neg mode 2 le {0.5 mul} if rlineto stroke} if\n"
+    "     } loop\n"
     "    } if} {mode 2 le intx 1 gt and {\n"
     "     /xm x def intx 1 sub {\n"
     "      /xm xtic intx div xm add def xm xmax ge {exit} if\n"
-    "      xm xTop {ymax} {ymin} ifelse mv 0 dy xTop {neg} if 0.5 mul\n"
-    "      rlineto stroke\n"
+    "      xBottom {xm ymin mv 0 dy 0.5 mul rlineto stroke} if\n"
+    "      xTop {xm ymax mv 0 dy -0.5 mul rlineto stroke} if\n"
     "     } repeat\n"
-    "    } if /x x xtic add def x xmax ge {exit} if x xTop {ymax} {ymin}\n"
-    "    ifelse mv 0 dy xTop {neg} if rlineto stroke\n"
-    "   } ifelse x xmax ge {exit} if} loop /y ymin def {logy\n"
+    "    } if x xmin gt {\n"
+    "     xBottom {x ymin mv 0 dy rlineto stroke} if\n"
+    "     xTop {x ymax mv 0 dy neg rlineto stroke} if\n"
+    "    } if /x x xtic add def\n"
+    "   } ifelse x xmax ge {exit} if} loop\n"
+    "  mode 2 le inty 1 gt and yOff 0 gt and logy not and {\n"
+    "   /d ytic inty div def /ym ymin yOff add d sub def {\n"
+    "    ym ymin le {exit} if\n"
+    "    yLeft {xmin ym mv dx 0.5 mul 0 rlineto stroke} if\n"
+    "    yRight {xmax ym mv dx -0.5 mul 0 rlineto stroke} if\n"
+    "    /ym ym d sub def} loop} if\n"
+    "   /y ymin yOff add def {logy\n"
     "   {/ya y def /y y ytic incLog def\n"
-    "    y ymax lt {yRight {xmax} {xmin} ifelse y mv dx yRight {neg} if\n"
-    "    0 rlineto stroke} if inty 1 gt {\n"
+    "    y ymax lt {yLeft {xmin y mv dx 0 rlineto stroke} if\n"
+    "    yRight {xmax y mv dx neg 0 rlineto stroke} if} if inty 1 gt {\n"
     "     /yd 10 ya mant sub exp def {/ya 10 ya exp yd add log def\n"
-    "      ya y ge ya ymax ge or {exit} if yRight {xmax} {xmin} ifelse\n"
-    "      ya mv dx yRight {neg} if\n"
-    "      mode 2 le {0.5 mul} if 0 rlineto stroke} loop\n"
+    "      ya y ge ya ymax ge or {exit} if\n"
+    "      yLeft {xmin ya mv dx mode 2 le {0.5 mul} if 0 rlineto stroke} if\n"
+    "      yRight {xmax ya mv dx neg mode 2 le {0.5 mul} if 0 rlineto stroke} if\n"
+    "     } loop\n"
     "    } if} {mode 2 le inty 1 gt and {\n"
     "     /ym y def inty 1 sub {\n"
     "      /ym ytic inty div ym add def ym ymax ge {exit} if\n"
-    "      yRight {xmax} {xmin} ifelse ym mv dx yRight {neg} if 0.5 mul 0\n"
-    "      rlineto stroke\n"
-    "     } repeat\n"
-    "    } if /y y ytic add def y ymax ge {exit} if yRight {xmax} {xmin}\n"
-    "    ifelse y mv dx yRight {neg} if 0 rlineto stroke\n"
+    "      yLeft {xmin ym mv dx 0.5 mul 0 rlineto stroke} if\n"
+    "      yRight {xmax ym mv dx -0.5 mul 0 rlineto stroke} if\n"
+    "     } repeat} if y ymin gt {\n"
+    "     yLeft {xmin y mv dx 0 rlineto stroke} if\n"
+    "     yRight {xmax y mv dx neg 0 rlineto stroke} if\n"
+    "    } if /y y ytic add def\n"
     "   } ifelse y ymax ge {exit} if} loop\n"
     "  mode 2 eq mode 4 eq or {\n"
-    "   colframe scol /y xTop {ymax} {ymin} ifelse xmax xmin sub\n"
-    "   relSize mul xscal mul yscal div xTop {0.025 mul neg} {0.05 mul}\n"
-    "   ifelse sub def /e xmax 2.99 gt xmin -2.99 lt or xtic 9.9 gt and\n"
-    "   logx and def /nd ndigx def /x xmin def {\n"
-    "    x xmax gt {/x xmax def} if\n"
-    "    e {(10#nu) x round cvi 4 string cvs strcat x y r2r 2 text}\n"
-    "     {logx {x 0 le {/nd 1 x 0.01 add cvi sub def} {/nd -1 def} ifelse}\n"
-    "     if x y mv x logx {10 exch exp} if nd prenum showc} ifelse\n"
-    "    logx {/x x xtic incLog def} {/x x xtic add def} ifelse\n"
-    "    x xtic 1e-3 mul xmax add gt {exit} if\n"
-    "   } loop /x yRight {xmax} {xmin} ifelse xmax xmin sub relSize mul\n"
-    "   0.025 mul yRight {neg} if sub def\n"
+    "   xBottom {/yB ymin xmax xmin sub relSize mul xscal mul yscal div\n"
+    "    0.05 mul sub def} if\n"
+    "   xTop {/yT ymax xmax xmin sub relSize mul xscal mul yscal div\n"
+    "    0.025 mul neg sub def} if\n"
+    "   /e xmax 2.99 gt xmin -2.99 lt or xtic 9.9 gt and\n"
+    "   logx and def /nd ndigx def /x xmin xOff add def /xe xmax xmin sub\n"
+    "   1e-3 mul xmax add def {\n"
+    "    x xmax gt {/x xmax def} if e {(10#nu) x round cvi 4 string cvs strcat\n"
+    "     xBottom {dup x yB r2r 2 text} if\n"
+    "     xTop {x yT r2r 2 text} {pop} ifelse}\n"
+    "     {logx {x 0 lt {/nd 1 x 0.01 add cvi sub def} {/nd -1 def} ifelse}\n"
+    "     if x logx {10 exch exp} if nd prenum colframe scol\n"
+    "     xBottom {dup x yB mv showc} if xTop {x yT mv showc} {pop} ifelse}\n"
+    "    ifelse logx {/x x xtic incLog def} {/x x xtic add def} ifelse\n"
+    "    x xe gt {exit} if\n"
+    "   } loop\n"
+    "   yLeft {/xL xmin xmax xmin sub relSize mul 0.025 mul sub def} if\n"
+    "   yRight {/xR xmax xmax xmin sub relSize mul -0.025 mul sub def} if\n"
     "   /dy xmax xmin sub 0.0125 mul relSize mul xscal mul yscal div def\n"
     "   /e ymax 2.99 gt ymin -2.99 lt or ytic 9.9 gt and logy and def\n"
-    "   /nd ndigy def /y ymin def {\n"
-    "    y ymax gt {/y ymax def}\n"
-    "    e {(10#nu) y round cvi 4 string cvs strcat x y dy sub r2r yRight\n"
-    "    {1} {3} ifelse text}\n"
-    "     {logy {y 0 le {/nd 1 y 0.01 add cvi sub def} {/nd -1 def} ifelse}\n"
-    "     if x y dy sub mv y logy {10 exch exp} if nd prenum\n"
-    "     yRight {show} {showr} ifelse} ifelse\n"
+    "   /nd ndigy def /y ymin yOff add def /ye ymax ymin sub 1e-3 mul\n"
+    "   ymax add def {\n"
+
+    "    y ymax gt {/y ymax def} if\n"
+    "    e {(10#nu) y round cvi 4 string cvs strcat\n"
+    "     yLeft {dup xL y dy sub r2r 3 text} if\n"
+    "     yRight {xR y dy sub r2r 1 text} {pop} ifelse}\n"
+    "     {logy {y 0 lt {/nd 1 y 0.01 add cvi sub def} {/nd -1 def} ifelse}\n"
+    "     if y logy {10 exch exp} if nd prenum colframe scol\n"
+    "     yLeft {dup xL y dy sub mv showr} if\n"
+    "     yRight {xR y dy sub mv show} {pop} ifelse}\n"
+    "    ifelse\n"
     "    logy {/y y ytic incLog def} {/y y ytic add def} ifelse\n"
-    "    y ytic 1e-3 mul ymax add gt {exit} if\n"
+    "    y ye gt {exit} if\n"
     "   } loop\n"
     "  } if\n"
-    " } if colframe scol 0.00417 slw frame coldata scol 0.00295 slw\n"
+    " } if colframe scol 0.00417 slw frame coldata scol 0.00295 slw} if\n"
     "} bind def\n\n"
     << "/text {\n"
     " /iorg ed gsave translate dir rotate 0 0 moveto /str ed\n"
@@ -325,17 +305,23 @@ void PSGraph::psInit(QTextStream* t, double xf, double yf)
     << "/arrow {\n"
     " colframe scol gsave scalx mul exch scalx mul exch translate newpath\n"
     " 0 0 moveto rotate -0.03333 rel dup dup\n"
-    " 0.2 mul dup neg 3 1 roll lineto lineto closepath fill dup 0 ne {\n"
-    "  0.00417 slw newpath scalx mul neg 0 moveto -0.03333 rel 0 lineto\n"
-    "  closepath stroke 0.00295 slw\n"
+    " 0.2 mul dup neg 3 1 roll lineto lineto closepath fill scalx mul dup\n"
+    " 0.03333 rel ge {\n"
+    "  0.00417 slw neg 0 moveto -0.03333 rel 0 lineto closepath stroke\n"
+    "  0.00295 slw\n"
     " } {pop} ifelse grestore coldata scol} bind def\n\n"
+    << "/arccm {\n"
+    " gsave scalx mul exch scalx mul exch translate rotate /h ed h div\n"
+    " /a ed /wl ed /wa ed 6 array currentmatrix a 1 scale newpath 0 0 h\n"
+    " 0.5 mul scalx mul wa wl wa add arc setmatrix stroke grestore\n"
+    " } bind def\n\n"
     << "/rectcm {\n"
-    " scalx mul /y exch def scalx mul /x exch def scalx mul y add exch\n"
-    " scalx mul x add exch x y rect {fill} {stroke} ifelse\n"
+    " gsave scalx mul exch scalx mul exch translate rotate scalx mul exch\n"
+    " scalx mul exch 0 0 rect {fill} {stroke} ifelse grestore\n"
     " } bind def\n\n"
     << "/ellcm {\n"
-    " gsave scalx mul exch scalx mul exch translate /h exch def h div\n"
-    " /a exch def 6 array currentmatrix a 1 scale newpath 0 0 h 0.5 mul\n"
+    " gsave scalx mul exch scalx mul exch translate rotate /h ed h div\n"
+    " /a ed 6 array currentmatrix a 1 scale newpath 0 0 h 0.5 mul\n"
     " scalx mul 0 360 arc setmatrix {fill} {stroke} ifelse grestore\n"
     " } bind def\n\n"
     << "/letH {\n"
@@ -360,19 +346,26 @@ void PSGraph::psInit(QTextStream* t, double xf, double yf)
     << "/letY {2 letXy} bind def\n\n"
     << "/plarry {\n"
     " gsave {psclip} if 3 1 roll /fy ed /fx ed symbol 0 gt {\n"
-    "  symbol lintyp 3 1 roll fy mul logy {log} if exch fx mul logx {log} if\n"
-    "  exch newpath r2r /y ed /x ed x y moveto 1 sub {\n"
-    "   fy mul logy {log} if exch fx mul logx {log} if exch r2r dup y sub\n"
-    "   dup mul 2 index x sub dup mul add sqrt\n"
-    "   dpos add /dpos ed dup /y ed /x 2 index def lineto\n"
+    "  symbol lintyp 3 1 roll fy mul /ya ed fx mul /xa ed /p1 false def\n"
+    "  xa 0 gt logx not or ya 0 gt logy not or and {\n"
+    "   xa logx {log} if ya logy {log} if newpath r2r /y ed /x ed\n"
+    "   x y moveto /p1 true def\n"
+    "  } if 1 sub {\n"
+    "   fy mul /ya ed fx mul /xa ed\n"
+    "   xa 0 gt logx not or ya 0 gt logy not or and {\n"
+    "    xa logx {log} if ya logy {log} if r2r dup y sub  dup mul\n"
+    "    2 index x sub dup mul add sqrt dpos add /dpos ed dup /y ed\n"
+    "    /x 2 index def p1 {lineto} {moveto /p1 true def} ifelse\n"
+    "   } if\n"
     "  } repeat stroke /dpos dpos dup 0.05 rel div floor 0.05 rel mul\n"
     "  sub def [] dpos setdash\n"
     " } {\n"
     "  /m [/m0 /m1 /m2 /m3 /m4 /m5 /m6 /m7 /m8 /m9 /m10 /m0 /m2 /m3 /m4\n"
     "  /m5 /m6] symbol neg 17 mod get load def {\n"
-    "   symbol -17 eq {pop pop} {fy mul logy {log} if exch fx mul logx {log}\n"
-    "   if exch m symbol neg 17 mod 10 le {stroke} {fill} ifelse} ifelse\n"
-    "  } repeat\n"
+    "   symbol -17 eq {pop pop} {fy mul /ya ed fx mul /xa ed xa 0 gt\n"
+    "   logx not or ya 0 gt logy not or and {xa logx {log} if ya logy\n"
+    "   {log} if m symbol neg 17 mod 10 le {stroke} {fill} ifelse} if}"
+    " ifelse\n  } repeat\n"
     " } ifelse grestore\n"
     "} bind def\n\n"
     << "/plerry {\n"
@@ -380,10 +373,10 @@ void PSGraph::psInit(QTextStream* t, double xf, double yf)
     " grestore\n} bind def\n\n"
     << "%%Page: 1 1\n\n"
     "gsave\n"
-    "/Helvetica /latin1 encfont findfont 0.033333 scf\n"
-    "/Helvetica-Bold /l1B encfont pop\n"
-    "/Helvetica-Oblique /l1O encfont pop\n"
-    "/Helvetica-BoldOblique /l1I encfont pop\n"
+    "/latin1 /Helvetica encfont findfont 0.033333 scf\n"
+    "/l1B /Helvetica-Bold encfont pop\n"
+    "/l1O /Helvetica-Oblique encfont pop\n"
+    "/l1I /Helvetica-BoldOblique encfont pop\n"
     "1 setlinecap 1 setlinejoin\n\n"
     "0.00295 slw\n\n%User code starts here\n\n";
 }
@@ -416,16 +409,16 @@ void PSGraph::Window(double x0a, double xla, double y0a, double yla)
   *ts << s.sprintf("%.2f %.2f %.2f %.2f window\n", x0, xl, y0, yl);
 }
 
-void PSGraph::scale(double xl, double xr, double yb, double yt,
+void PSGraph::scale(double _xl, double xr, double yb, double yt,
                     bool lgx, bool lgy)
 {
   logx = lgx;
   logy = lgy;
   if (logx) {
-    xmin = log10(xl);
+    xmin = log10(_xl);
     xmax = log10(xr);
   } else {
-    xmin = xl;
+    xmin = _xl;
     xmax = xr;
   }
   if (logy) {
@@ -435,7 +428,7 @@ void PSGraph::scale(double xl, double xr, double yb, double yt,
     ymin = yb;
     ymax = yt;
   }
-  *ts << QString::number(xl) << " " << QString::number(xr) << " "
+  *ts << QString::number(_xl) << " " << QString::number(xr) << " "
       << QString::number(yb) << " " << QString::number(yt)
       << (logx ? " true" : " false") << (logy ? " true" : " false")
       << " psscale\n";
@@ -450,19 +443,19 @@ void PSGraph::setDig(int nx, int ny)
 void PSGraph::setColFrame(unsigned icol)
 {
   QCString s;
-  *ts << s.sprintf("/colframe 16#%x def\n", Utils::rgbQt1(icol));
+  *ts << s.sprintf("/colframe 16#%x def\n", rgbQt1(icol));
 }
 
 void PSGraph::setColGrid(unsigned icol)
 {
   QCString s;
-  *ts << s.sprintf("/colgrid 16#%x def\n", Utils::rgbQt1(icol));
+  *ts << s.sprintf("/colgrid 16#%x def\n", rgbQt1(icol));
 }
 
 void PSGraph::setColData(unsigned icol)
 {
   QCString s;
-  *ts << s.sprintf("/coldata 16#%x def\ncoldata scol\n",  Utils::rgbQt1(icol));
+  *ts << s.sprintf("/coldata 16#%x def\ncoldata scol\n",  rgbQt1(icol));
 }
 
 void PSGraph::setCol(unsigned icol1, unsigned icol2, unsigned icol3)
@@ -478,11 +471,14 @@ void PSGraph::frame(void)
 }
 
 void PSGraph::raster(double xtic, double ytic, int intx, int inty, int mode,
-                     bool xTop, bool yRight)
+                     double xOff, double yOff, bool xBottom, bool xTop,
+                     bool yLeft, bool yRight)
 {
   *ts << QString::number(xtic) << " " << QString::number(ytic) << " "
       << QString::number(intx) << " " << QString::number(inty) << " "
-      << QString::number(mode) << (xTop ? " true" : " false")
+      << QString::number(mode) << " " << QString::number(xOff) << " "
+      << QString::number(yOff) << (xBottom ? " true" : " false")
+      << (xTop ? " true" : " false") << (yLeft ? " true" : " false")
       << (yRight ? " true" : " false") << " raster\n";
 }
 
@@ -509,20 +505,31 @@ void PSGraph::arrow(double x, double y, double dir, double len, QRect*)
       << QString::number(x) << " " << QString::number(y) << " arrow\n";
 }
 
+void PSGraph::arc(double x, double y, double w, double h, double a,
+                  double alen, double dir, QRect*)
+{
+  *ts << QString::number(a) << " " << QString::number(alen) << " "
+      << QString::number(w) << " " << QString::number(h) << " "
+      << QString::number(dir) << " " << QString::number(x) << " "
+      << QString::number(y) << " arccm\n";
+}
+
 void PSGraph::rectangle(double x, double y, double w, double h,
-                        bool filled, QRect*)
+                        bool filled, double dir, QRect*)
 {
   *ts << (filled ? "true " : "false ")
       << QString::number(w) << " " << QString::number(h) << " "
-      << QString::number(x) << " " << QString::number(y) << " rectcm\n";
+      << QString::number(dir) << " " << QString::number(x) << " "
+      << QString::number(y) << " rectcm\n";
 }
 
 void PSGraph::ellipse(double x, double y, double w, double h,
-                     bool filled, QRect*)
+                     bool filled, double dir, QRect*)
 {
   *ts << (filled ? "true " : "false ")
       << QString::number(w) << " " << QString::number(h) << " "
-      << QString::number(x) << " " << QString::number(y) << " ellcm\n";
+      << QString::number(dir) << " " << QString::number(x) << " "
+      << QString::number(y) << " ellcm\n";
 }
 
 void PSGraph::letH(const QString& str, bool xTop)
@@ -557,22 +564,28 @@ void PSGraph::plArray(const double* x, const double* y,
   if (n > 0) {
     int ie = 0;
     int ia;
+    bool xNorm = (fx > 1.0e33) || (fx < 1.0e-33);
+    bool yNorm = (fy > 1.0e33) || (fy < 1.0e-33);
     do {
       ia = QMIN(n - 1, ie + 127);
       *ts << "\n";
       int fail = 0;
       QCString s;
       for (int i = ia; i >= ie; i--) {
-        s.sprintf("%g %g\n", x[i], y[i]);
+        s.sprintf("%g %g\n", xNorm ? (fx * x[i]) : x[i],
+                  yNorm ? (fy * y[i]) : y[i]);
         if (s.find("nan") != -1) {
           s.prepend("%%");
           fail++;
         }
         *ts << s;
       }
-      *ts << QString::number(fx) << " " << QString::number(fy) << " "
-          << QString::number(ia - ie - fail + 1) << (clip ? " true" : " false")
-          << " plarry\n";
+      int np = ia - ie - fail + 1;
+      if (np)
+        *ts << QString::number(xNorm ? 1.0 : fx) << " "
+            << QString::number(yNorm ? 1.0 : fy) << " "
+            << QString::number(np) << (clip ? " true" : " false")
+            << " plarry\n";
       if (kSymb > 0)
         ie = ia;
       else
@@ -587,26 +600,36 @@ void PSGraph::plError(const double* x, const double* y, const double* e,
   if (n > 0) {
     int ie = 0;
     int ia;
+    bool xNorm = (fx > 1.0e33) || (fx < 1.0e-33);
+    bool yNorm = (fy > 1.0e33) || (fy < 1.0e-33);
     do {
       ia = QMIN(n - 1, ie + 127);
       *ts << "\n";
       int fail = 0;
       QCString s;
       for (int i = ia; i >= ie; i--) {
-        s.sprintf("%g %g %g\n", x[i], y[i], e[i]);
+        s.sprintf("%g %g %g\n", xNorm ? (fx * x[i]) : x[i],
+                  yNorm ? (fy * y[i]) : y[i],
+                  yNorm ? (fy * e[i]) : e[i]);
         if (s.find("nan") != -1) {
           s.prepend("%%");
           fail++;
         }
         *ts << s;
       }
-      *ts << QString::number(fx) << " " << QString::number(fy) << " "
+      *ts << QString::number(xNorm ? 1.0 : fx) << " "
+          << QString::number(yNorm ? 1.0 : fy) << " "
           << QString::number(ia - ie - fail + 1) << " plerry\n";
       ie = ia + 1;
     } while (ia != (n - 1));
   }
 }
 
+void PSGraph::resetDash()
+{
+  *ts << "/dpos 0 def\n";
+}
+
 void PSGraph::psExit(void)
 {
   *ts << "\nshowpage grestore\n\n%%Trailer\n%%EOF\n";
@@ -625,8 +648,7 @@ void PSGraph::psStr(const char* str)
         case 40:
         case 41:
         case 92:
-          *ts << s.sprintf("\\%03o", *c);
-          break;
+          *ts << "\\";
         default:
           *ts << QChar(*c);
       }
@@ -635,13 +657,13 @@ void PSGraph::psStr(const char* str)
   *ts << ")";
 }
 
-void PSGraph::cm2r(double xcm, double ycm, double fxn, double fyn,
+void PSGraph::cm2r(double xcm, double ycm, double _fxn, double _fyn,
                    double* xr, double* yr) const
 {
-  *xr = xmin + (xcm - x0) * fxn * (xmax - xmin) / xl;
+  *xr = xmin + (xcm - x0) * _fxn * (xmax - xmin) / xl;
   if (logx)
     *xr = pow(10, *xr);
-  *yr = ymin + (ycm - y0) * fyn * (ymax - ymin) / yl;
+  *yr = ymin + (ycm - y0) * _fyn * (ymax - ymin) / yl;
   if (logy)
     *yr = pow(10, *yr);
 }
